@*                                               *****************************************************************
@*                                               * FILE:        computeHamiltonian.prc                           *
@*                                               * written by:  Alexander Hentschel                              *
@*                                               *              Humboldt University Berlin                       *
@*                                               *              Alexander.Hentschel@physik.hu-berlin.de          *
@*                                               *              September 2006                                   *
@*                                               ****************************************************************{


@*****************************************************************************************************************
@*************************************   Module computeH_procedures   ********************************************
@*****************************************************************************************************************
@* provides the following FORM procedures:			 						 *
@*    SetNotUsedOperatorClassesToZero(Operator,Function1,Function2,Function3,Function4)				 *
@*    DropNotUsedOperators(Operator,Function1,Function2,Function3,Function4)					 *
@*    SubstituteModeDecomposition										 *
@*    HamiltonianSegmentation											 *
@*    checkHamiltonianSegmentation										 *
@*    RewriteOperators(Operator)										 *
@*    											 *
@*    											 *
@* ------------------------------------------------------------------------------------------------------------- *
@*****************************************************************************************************************

@*********************************************  global definitions  **********************************************
  CFunction Ydeg, Ydegprime, Zdeg, Zdegprime;
  CFunction BetaTildePlus, BetaTildePlusPrime, BetaTildeMinus, BetaTildeMinusPrime;
  CFunction AlphaTildePlus, AlphaTildePlusPrime, AlphaTildeMinus, AlphaTildeMinusPrime; 
  NFunction EtaTildeMinusPrime, EtaTildeMinus, EtaTildePlusPrime, EtaTildePlus;
  NFunction ThetaTildeMinusPrime, ThetaTildeMinus, ThetaTildePlusPrime, ThetaTildePlus;
  CFunction BetaPlus, BetaMinus, AlphaPlus, AlphaMinus;
  NFunction EtaMinus, EtaPlus, ThetaMinus, ThetaPlus; 
  set FermionicTildeFields :	EtaTildePlus, EtaTildeMinus, ThetaTildePlus, ThetaTildeMinus,
  				EtaTildePlusPrime, EtaTildeMinusPrime, ThetaTildePlusPrime, ThetaTildeMinusPrime;
  CFunction gmTheta, gmEta, OperatorContainer, ModeIndexContainer, Exp;
  Symbols DummyIndex;
  #define CalculationAbort "0"
@*****************************************  end of global Definitions *******************************************}


@********************************* Procedure SetNotUsedOperatorClassesToZero ************************************{
@* USE: 	SetNotUsedOperatorClassesToZero(Operator,Function1,Function2,Function3,Function4)		 *
@* DESCRIPTION:							 						 *
@*   If "Operator"1 up to "Operator"4 are all not defined (so there is no appearance of this operator class 	 *
@*   at all), the functions of this operators given by Function1,...,Function4 are set to zero, 	 	 *
@*   since they generate (implicit normal ordered) terms of this operators but the states are vacuum states 	 *
@*   refered to this operator.											 *
@*****************************************************************************************************************
#procedure SetNotUsedOperatorClassesToZero(Operator,Function1,Function2,Function3,Function4)
   #ifndef ``Operator'1'
     #ifndef ``Operator'2'
	#ifndef ``Operator'3'
	  #ifndef ``Operator'4'
	     #define No'Operator'
	     #ifdef `VERBOSE'
	       #write "   MESSAGE: No 'Operator'-Terms found"
	     #endif
	     id 'Function1'(DummyIndex?) = 0;
	     id 'Function2'(DummyIndex?) = 0;
	     id 'Function3'(DummyIndex?) = 0;
	     id 'Function4'(DummyIndex?) = 0;
	     
	     id 'Function1' = 0;
	     id 'Function2' = 0;
	     id 'Function3' = 0;
	     id 'Function4' = 0;
	  #endif  
	#endif  
     #endif  
   #endif  
#endprocedure
@*********************************  End of SetNotUsedOperatorClassesToZero  *************************************}

@************************************** Procedure DropNotUsedOperators ******************************************{
@* USE: 	DropNotUsedFermionicOperators(Operator,Function1,Function2,Function3,Function4)			 *
@* DESCRIPTION:							 						 *
@*   For every "Operator"1 ... "Operator"4 which is not defined (states act as vacuum referred to this operator) *
@*   the functions of this operators given by Function1,...,Function4 are set to zero, since they generate	 *
@*   (implicit normal ordered) terms acting on a vacuum states refered to this operator. 			 *
@*****************************************************************************************************************

#procedure DropNotUsedOperators(Operator,Function1,Function2,Function3,Function4)
   #do lvar = 1,4
      #ifndef ``Operator'`lvar''
	     id 'Function1'(`lvar',?Ia) = 0;
	     id 'Function2'(`lvar',?Ia) = 0;
	     id 'Function3'(`lvar',?Ia) = 0;
	     id 'Function4'(`lvar',?Ia) = 0;
      #endif
   #enddo
#endprocedure
@****************************************  End of DropNotUsedOperators  *****************************************}

@********************************** Procedure SubstituteModeDecomposition ***************************************{
@* USE: 	SubstituteModeDecomposition									 *
@* DESCRIPTION:							 						 *
@*   Substitutes for ThetaTildeMinus,ThetaTildeMinusPrime,...,AlphaTildePlusPrime,AlphaTildePlus the appropriate *
@*   mode decomposition. Thereby the mode names ModeIndex1,...,ModeIndex4 are used. The substitution is done	 *
@*   with priority to the "convenient normal oder", i.e. ThetaTildeMinus is substituted first if there is an 	 *
@*   appearance in the current term and AlphaTildePlus last. The first substitution is done by using ModeIndex1	 *
@*   and the last by using ModeIndex4, where summation over ModeIndex1,...,ModeIndex4 is implied		 *
@*****************************************************************************************************************
#procedure FerminonicSubstModeDecomposition(Operator)
 #do Flavour  = {4,3,2,1}
   #do Kind = {Plus,Minus}
	repeat;
	  if (match( `Operator'Tilde`Kind'(`Flavour',DF?) ));
	    id once `Operator'Tilde`Kind'(`Flavour',DF?)   
       		=   Exp(ModeIndexContainer($Counter))*DF(ModeIndexContainer($Counter))
		  * `Operator'`Kind'(`Flavour',``Kind''ModeIndexContainer($Counter)) * gm`Operator'(`Flavour');
	    $Counter = $Counter + 1;
	  endif;
	endrepeat;
	repeat;
	  if (match( `Operator'Tilde`Kind'Prime(`Flavour',DF?) ));
	    id once `Operator'Tilde`Kind'Prime(`Flavour',DF?)   
       		=   KomplexI*ModeIndexContainer($Counter)*Exp(ModeIndexContainer($Counter))*DF(ModeIndexContainer($Counter))
		  * `Operator'`Kind'(`Flavour',``Kind''ModeIndexContainer($Counter)) * gm`Operator'(`Flavour');
	    $Counter = $Counter + 1;
	  endif;
	endrepeat;	
   #enddo
 #enddo
#endprocedure

#procedure BosonicSubstModeDecomposition(Operator)
 #do Flavour  = {4,3,2,1}
   #do Kind = {Plus,Minus}
	repeat;
	  if (match( `Operator'Tilde`Kind'(`Flavour',Ib?) ));
	    id once `Operator'Tilde`Kind'(`Flavour',Ib?)   
	      =   Exp(ModeIndexContainer($Counter))*Omega(ModeIndexContainer($Counter),Ib/2)
	        * `Operator'`Kind'(`Flavour',``Kind''ModeIndexContainer($Counter));
	    $Counter = $Counter + 1;
	  endif;
	endrepeat;
	repeat;
	  if (match( `Operator'Tilde`Kind'Prime(`Flavour',Ib?) ));
	    id once `Operator'Tilde`Kind'Prime(`Flavour',Ib?)   
	      =   KomplexI*ModeIndexContainer($Counter)*Exp(ModeIndexContainer($Counter))*Omega(ModeIndexContainer($Counter),Ib/2)
	        * `Operator'`Kind'(`Flavour',``Kind''ModeIndexContainer($Counter));
	    $Counter = $Counter + 1;
	  endif;
	endrepeat;	
   #enddo
 #enddo
#endprocedure
  
#procedure SubstituteModeDecomposition
 #define Plus ""
 #define Minus "-"
 #define NameAddOnPlus "p"
 #define NameAddOnMinus "m"
 term;
   $Counter = 1;
    #ifndef `NoTheta'
      #call FerminonicSubstModeDecomposition(Theta)
    #endif
    #ifndef `NoEta'
      #call FerminonicSubstModeDecomposition(Eta)
    #endif
    #ifndef `NoBeta'
      #call BosonicSubstModeDecomposition(Beta)
    #endif
    #ifndef `NoAlpha'
      #call BosonicSubstModeDecomposition(Alpha)
    #endif
 endterm;
 #do i = 1,`LengthOfOperatorWord'
    id many ModeIndexContainer(`i') = ModeIndex`i';
 #enddo
 argument;
   #do i = 1,`LengthOfOperatorWord'
      id many ModeIndexContainer(`i') = ModeIndex`i';
   #enddo
 endargument;
 id KomplexI^4 = 1;
 id KomplexI^2 = -1; 
#endprocedure
@*************************************  End of SubstituteModeDecomposition  *************************************}

@************************************* Procedure HamiltonianSegmentation ****************************************{
@* USE:  generateHamiltonianParts						 				 *
@* DESCRIPTION:							 						 *
@*    The function splits the Hamiltonian in a sum of the type							 *
@*	H =        HamiltonianFunctionPart1 * HamiltonianOperatorPart1 						 *
@*	   + ... + HamiltonianFunctionPartN * HamiltonianOperatorPartN						 *
@*    where every HamiltonianOperatorPart'i' is a singular product of 4 operators. Every possible combination	 *
@*    of the operators is checke, whether it contributes to the Hamiltonian. If and only if a		  	 *
@*    local FORM expression is generated containing the coefficient of this operator product and another 	 *
@*    local FORM expression containing the operator product itself. 						 *
@*****************************************************************************************************************
#procedure HamiltonianSegmentation
  Hide;
  #$PartCounter = 0;
  Local TempStep0 =   [Hbb] + [Hff-Part1] + [Hff-Part2]
	    	    + [Hbf-Part1]+[Hbf-Part2]+[Hbf-Part3]+[Hbf-Part4]+[Hbf-Part5]+[Hbf-Part6];
  Local [CalculationAbort] = termsin_(TempStep0)*OperatorContainer() + delta_(termsin_(TempStep0));		    
  id NDF1?AllOperators(ModeIndex1?)*NDF2?AllOperators(ModeIndex2?)*NDF3?AllOperators(ModeIndex3?)*NDF4?AllOperators(ModeIndex4?) 
          = OperatorContainer( NDF1(ModeIndex1) * NDF2(ModeIndex2) * NDF3(ModeIndex3) * NDF4(ModeIndex4) ); 
  if ( match(OperatorContainer(?Ia)) == 0 );
  	redefine CalculationAbort "1";
  endif;
  .sort
  Drop [CalculationAbort];
  #if (`CalculationAbort' == 1)
     #write "ERROR: Hamiltonian = 0 with given input!"
     #write "       calculation terminated!"
     #terminate 1;
  #endif
  .sort
  #do foundTerm = 1,1
     $PartCounter = {`$PartCounter' +1};   
     .sort
     Hide;
     Local HamiltonianFunctionPart`$PartCounter' = TempStep{`$PartCounter'-1};
     if (match(OperatorContainer(?Ia$O1))); 
     endif;
    .sort
     if (match(OperatorContainer($O1)));
         redefine foundTerm "0";
         id  OperatorContainer(?Ia) = 1;
     else;
         Discard;
     endif;
    .sort
     Unhide;
    .sort
     Drop TempStep{`$PartCounter'-1};
     Local HamiltonianOperatorPart`$PartCounter' = `$O1';
     Local TempStep`$PartCounter' = TempStep{`$PartCounter'-1} - HamiltonianFunctionPart`$PartCounter' *  OperatorContainer(`$O1');
     #ifdef `VERBOSE'
       #if (`foundTerm' == 0)
          #write  "            contributing operator product: `$O1'  -- term Nr: `$PartCounter'"
       #endif
     #endif
  #enddo
  .sort
  Drop HamiltonianFunctionPart`$PartCounter';
  Drop HamiltonianOperatorPart`$PartCounter';
  Drop TempStep`$PartCounter';  
  $PartCounter = {`$PartCounter' -1};
  .sort
 #write  "            `$PartCounter' different kinds of operator terms"
#endprocedure
@**************************************  End of HamiltonianSegmentation  ****************************************}

@********************************** Procedure checkHamiltonianSegmentation **************************************{
@* USE: checkHamiltonianSegmentation										 *
@* DESCRIPTION:							 						 *
@*    This function checks, whether the segementation of the Hamiltonian reproduces the full Hamiltonian, 	 *
@*    i.e. if the sum of all HamiltonianFunctionPart * HamiltonianOperatorPart equals the original Hamiltonian.	 *
@*    It is also checked that every HamiltonianOperatorPart consists only of one term.				 *
@*****************************************************************************************************************
#procedure checkHamiltonianSegmentation 
  Local Rest0 = [Hbb]+[Hff-Part1]+[Hff-Part2]+[Hbf-Part1]+[Hbf-Part2]+[Hbf-Part3]+[Hbf-Part4]+[Hbf-Part5]+[Hbf-Part6];
  #if (`$PartCounter' > 0)
    #do lvar = 1,`$PartCounter'
      .sort
      Drop Rest{`lvar'-1};
      Local Rest`lvar' = Rest{`lvar'-1} - HamiltonianFunctionPart`lvar' * HamiltonianOperatorPart`lvar';
    #enddo
  #endif
  .sort
  Drop Rest`$PartCounter';
  $checkHamiltonianSegmentationResult = Rest`$PartCounter';
  $r = 0;
  .sort
  #if ("`$checkHamiltonianSegmentationResult'" != "0")
     Print "";  Print "ERROR:  Internal bug - Hamiltonian segmentation leaves rest";   
		Print "        Please report bug to hentsche[at]physik.hu-berlin.de"; exit "";
  #endif
  #if (`$PartCounter' > 0)
    #do lvar = 1,`$PartCounter'
      #if (termsin(HamiltonianOperatorPart`lvar') != 1)
         #message in if
         Print "";  Print "ERROR:  Internal bug - HamiltonianOperatorPart`lvar' consists of more than one term";   
	            Print "        Please report bug to hentsche[at]physik.hu-berlin.de"; exit "";
      #endif
    #enddo
  #endif
  .sort
  #if ("`$checkHamiltonianSegmentationResult'" == "0")
     #write "   MESSAGE: Hamiltonian segmentation successful"
  #endif
#endprocedure
@************************************  End of checkHamiltonianSegmentation  *************************************}

@**************************** Procedure RewriteOperators & BackWriteOperators ***********************************{
@* USE: 	RewriteOperators(Operator) 									 *
@*		BackWriteOperators(Operator)									 *
@* DESCRIPTION:							 						 *
@*    RewriteOperators												 *
@*    	renames the operators in the following way: "Operator"Plus(F,Index) -> "Operator""F"Plus(Index)		 *
@*    	("Operator"Minus analog) where F denotes the operator 'flavour'. "Operator" specifies the operator class *
@*    	(Alpha, Theta, ...). 											 *
@*    BackWriteOperators											 *
@*	changes the names back into original style.					  			 *
@*****************************************************************************************************************
#procedure RewriteOperators(Operator)
 #do lvar = 1,4
   #ifdef ``Operator'`lvar''
     id many `Operator'Plus(`lvar', ?Ia) = Op`Operator'`lvar'p(?Ia);
     id many `Operator'Minus(`lvar', ?Ia) = Op`Operator'`lvar'm(?Ia);
   #endif
 #enddo
#endprocedure

#procedure BackWriteOperators(Operator)
 #do lvar = 1,4
   #ifdef ``Operator'`lvar''
     id many Op`Operator'`lvar'p(?Ia) = `Operator'Plus(`lvar', ?Ia);
     id many Op`Operator'`lvar'm(?Ia) = `Operator'Minus(`lvar', ?Ia);
   #endif
 #enddo
#endprocedure
@*******************************  End of RewriteOperators & BackWriteOperators  *********************************}

@*************************************** Procedure DropNonSectorParts *******************************************{
@* USE: DropNonSectorParts											 *
@* DESCRIPTION:							 						 *
@*   If a terms consists of more operators of one flavour than there are excitations in the states, the term	 *
@*   Is discarded. The number of excitations is defined by "HamiltonianSector" in the file "define_states.def"	 *
@*   If there is no definition of "HamiltonianSector" given, this function doesn't do anything.			 *
@*****************************************************************************************************************
#procedure DropNonSectorParts
  #ifdef `HamiltonianSectorDefined'
    #write "   MESSAGE: discarding Hamiltonian terms, not contributing to given HamiltonianSector"
    #do Operator = {Theta,Eta,Beta,Alpha}
      #do Flavour = 1,4
        #ifdef ``Operator'`Flavour''
           if (count(Op`Operator'`Flavour'p,1) > ``Operator'`Flavour'' ) Discard;
           if (count(Op`Operator'`Flavour'm,1) > ``Operator'`Flavour'' ) Discard;
        #endif
      #enddo
    #enddo
  #endif
  .sort  
#endprocedure
@****************************************  End of DropNonSectorParts  *******************************************}

@************************************** Procedure exportHamiltonianToForm ****************************************{
@* USE: exportHamiltonianToForm(FileName)									 *
@* DESCRIPTION:							 						 *
@* 			 *
@* 			 *
@* 			 *
@* 			 *
@* 			 *
@*****************************************************************************************************************
#procedure exportHamiltonianToForm(FileName)
   id gmTheta(Ia?) = 1;
   id gmEta(Ia?) = 1;
   id Exp(?Ia) = DeltaSum(?Ia);
   id Omega(Ia?, -1/2) = InvSqrtOmega(Ia);
   id Omega(Ia?, 1/2) = SqrtOmega(Ia);
   .sort
   format 130;
   #create <'FileName'>
   #write <'FileName'> "@*                                   ****************************************************************************"
   #write <'FileName'> "@*                                   * FILE:        `FileName'                                   *"
   #write <'FileName'> "@*                                   * computer generated output file by FORM-programm computeHamiltonian.prc   *"
   #write <'FileName'> "@*                                                           on `DATE_'"
   #write <'FileName'> "@*                                   ****************************************************************************"
   #write <'FileName'> " "
   #write <'FileName'> "@ permitted operators:   %"
   #do Operator = {Theta,Eta,Beta,Alpha}
     #ifndef `No`Operator''
        #do Flavour = {4,3,2,1}
           #ifdef ``Operator'`Flavour''
	      #write <'FileName'> "`Operator'`Flavour' %"
	   #endif
        #enddo
	#write <'FileName'> " "
	#write <'FileName'> "@                        %"
     #endif
   #enddo
   #write <'FileName'> " "
   #write <'FileName'> "#define HamiltonianParts \"`$PartCounter'\""
   #write <'FileName'> " "
   #do i=1,`$PartCounter'
     #write <'FileName'> "Local HamiltonianFunctionPart`i' = "
     #write <'FileName'> "      %e",HamiltonianFunctionPart`i'
     #write <'FileName'> "Local HamiltonianOperatorPart`i' = "
     #write <'FileName'> "      %e",HamiltonianOperatorPart`i'
   #enddo
   #write <'FileName'> " "
#endprocedure
@*************************************  End of eportHamiltonianToForm  ******************************************}

@**********************************************  END OF MODULE   ************************************************@
