/*                                               *****************************************************************
//                                               * FILE:        generateSectors.c                                *
//                                               * written by:  Alexander Hentschel                              *
//                                               *              Humboldt University Berlin                       *
//                                               *              Alexander.Hentschel@physik.hu-berlin.de          *
//                                               *              November 2006                                    *
//                                               ****************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <time.h>
#include <string.h>

#include "generateSectors.h"

#define DefaultOperatorOrder {"Theta4","Theta3","Theta2","Theta1","Eta4","Eta3","Eta2","Eta1","Beta4","Beta3","Beta2","Beta1","Alpha4","Alpha3","Alpha2","Alpha1"} 

static Type_Konf* Sectors = NULL;		/* pointing on first element of mixing sector list		*/
static int *Konf;				/* working configuration 					*/
static int NumberOfOperators;
static int NumberOfCharges;
static int *OpCharge;
static int *charges;
static int Anz = 0;

static int Operator_Charges_defined = 0;
static int needed_Charges_defined = 0; 
static char* OpOrder[] = DefaultOperatorOrder; 

/* Funktion today()
 * return value: String der aktuelles Datum und Uhrzeit enthaelt 								*/
static char* today(void) {
  struct tm *tmnow;
  time_t tnow;
  time(&tnow);
  tmnow = localtime(&tnow);
  return(asctime(tmnow));
} /* end of function today */

/* function define_default_Operator_charges()
 * DESCRIPTION: 
 * defines charges of every operator, where Operators are normal ordered    
 * normal ordering als well as charges are in convention of the paper hep-th/0603008 						*/
void define_default_Operator_charges(){
         /* S1 S2 J1 J2    S1 S2 J1 J2   S1 S2 J1 J2    S1 S2 J1 J2  					  */
 int c[] = {0, 0, 1, 1,    0, 0, 1,-1,   0, 0,-1, 1,    0, 0,-1,-1,     /* Alpha1, Alpha2, Alpha3, Alpha4 */   
 	    1, 1, 0, 0,    1,-1, 0, 0,  -1, 1, 0, 0,   -1,-1, 0, 0,     /* Beta1,  Beta2,  Beta3,  Beta4  */ 
	    1, 0, 0, 1,    1, 0, 0,-1,  -1, 0, 0, 1,   -1, 0, 0,-1,     /* Eta1,   Eta2,   Eta3,   Eta4   */   
	    0, 1, 1, 0,    0,-1, 1, 0,   0, 1,-1, 0,    0,-1,-1, 0,};   /* Theta1, Theta2, Theta3, Theta4 */  
 int op,ch;
 char* OpOrder[] = DefaultOperatorOrder;
 NumberOfCharges = 4; NumberOfOperators = sizeof(OpOrder)/sizeof(*OpOrder);  
 OpCharge = (int*) malloc(sizeof(int)*NumberOfCharges*NumberOfOperators);
 for(op=0; op <NumberOfOperators; op++)
   for(ch=0; ch <NumberOfCharges; ch++)
     OpCharge[op*4+ch] = c[NumberOfCharges*NumberOfOperators-1 - (op*4+ 3- ch)];
 Operator_Charges_defined = 1;
 needed_Charges_defined = 0; 
 printf("   MESSAGE: defining default charges\n");
}/* end of function define_Operator_charges() */

/* returns "1" if Op is a bosonic operator and "0" otherwise 
 * may be changed in future to meet needs of more general algorithms 
 * CAUTION: it needs to be able to deal with the value Op = -1, in this case the return value is arbitrarily ! 			*/
int isBosonic(int Op){
  if (Op > 7) return(1);
  return(0);
} /* end of function isBosonic() */

/* function createListElement(int NumOp)
 * DESCRIPTION:
 * This function creates a list element of the type Type_Konf.
 * NumOp defines the number of Elements in the integer list 'Sector'. The pointer 'next' is set to NULL.
 * A pointer (of type Type_Konf) to the created list element is returned. 							*/
Type_Konf* createListElement(int NumOp){
  Type_Konf* elm;
  elm = (Type_Konf*) malloc(sizeof(Type_Konf));
  if(elm == NULL) {printf("   MESSAGE: memory overflow - abort calculation, sorry\n"); exit(1);}
  if(NumOp == 0) elm->Sector = NULL;
  else {
     elm->Sector = (int*) malloc(sizeof(int)*NumOp);
     if((elm->Sector) == NULL) {printf("   MESSAGE: memory overflow - abort calculation, sorry\n"); exit(1);}
  }
  elm->next = NULL;
  return(elm);
} /* end of function createListElement() */

/* function create_list_header()
 * DESCRIPTION:
 * This function creates a list header of type Type_Mixing_Sectors and returns the corresponding pointer
 * The OperatorList and SectorList pointer are both set to zero.								*/
Type_Mixing_Sectors* create_list_header(){
  Type_Mixing_Sectors *Mixing_Sectors;
  Mixing_Sectors = (Type_Mixing_Sectors*) malloc(sizeof(Type_Mixing_Sectors));
  if(Mixing_Sectors == NULL) {printf("   MESSAGE: memory overflow\n"); exit(1);}
  Mixing_Sectors->OperatorList = NULL;
  Mixing_Sectors->SectorList = NULL;
  Mixing_Sectors->NumberOfSectors = Mixing_Sectors->NumberOfOperators = 0;
  return(Mixing_Sectors);
} /* end of function Type_Mixing_Sectors() */


/* function define_Sector_Operators(Type_Mixing_Sectors *Mixing_Sectors, char* Operators[], int NumOp)
 * DESCRIPTION:
 * This function copies the Operator list given by 'Operators' into the structure 'Mixing_Sectors' 
 * the old list 'Mixing_Sectors->OperatorList' is destroyed, and the occupied memory is freed  					*/
void define_Sector_Operators(Type_Mixing_Sectors *Mixing_Sectors, char* Operators[], int NumOp) {
  int i;
  char **OldOperatorList;
  if(Mixing_Sectors == NULL) return;
  OldOperatorList = Mixing_Sectors->OperatorList;				/* save adress of old operator list		*/
  Mixing_Sectors->OperatorList = (char**) malloc(sizeof(char*)*NumOp); 		/* memory for new list 				*/
  for(i=0; i<NumOp; i++) {							/* copy elements of 'Operators' into new list: 	*/
    Mixing_Sectors->OperatorList[i] = (char*) malloc(strlen(Operators[i])+1);		/* memory for new list element		*/
    strcpy(Mixing_Sectors->OperatorList[i], Operators[i]);				/* copy content   			*/
  } /* end of for(i=0; i<NumOp; i++) */
  /* free memory of old list, if there was one */
  if (OldOperatorList != NULL) for(i=0; i<Mixing_Sectors->NumberOfOperators; i++) free(OldOperatorList[i]);
  free(OldOperatorList);
  Mixing_Sectors->NumberOfOperators = NumOp;  				/* update number 'NumberOfOperators' to new value	*/
} /* end of function define_Sector_Operators() */

/* function free_Sector_List(Type_Konf* List)
 * DESCRIPTION:
 * frees memory of Sector list, created by generate_Sectors() or append_found_Sector() */
Type_Mixing_Sectors* free_Sector_List(Type_Mixing_Sectors *Mixing_Sectors){
  int i;
  if(Mixing_Sectors == NULL) return(Mixing_Sectors);
  { /* remove list of found sectors */
    Type_Konf* tmp = Mixing_Sectors->SectorList;
    Type_Konf* next;
    while(tmp != NULL){
      free(tmp->Sector);
      next = tmp->next;
      free(tmp);
      tmp = next;
    }
    for(i=0; i<Mixing_Sectors->NumberOfOperators; i++) free(Mixing_Sectors->OperatorList[i]);
    free(Mixing_Sectors->OperatorList);
  }
  free(Mixing_Sectors);
  return(NULL);
} /* end of function writeSector() */

/* function append_found_Sector(Type_Konf* Konf)
 * DESCRIPTION:
 * This function writes the found configuration into the list of found Sectors and retunrs a pointer to the new
 * entry created */
Type_Konf* append_found_Sector(int *Konf){
  static Type_Konf **lastSector;		/* designed to point on last 'next'-pointer in the list 	*/
  Type_Konf *tmp;
  int i;
  if(Sectors == NULL) { 
     lastSector= &(Sectors);			/* if list is empty use 'Sectors' as first "next"-pointer 	*/
     Anz = 0;
  }
  tmp = createListElement(NumberOfOperators);				/* create new list element 'tmp'	*/
  for(i=0; i<NumberOfOperators; i++) tmp->Sector[i] = Konf[i];		/* copy configuration to 'tmp' 		*/
  *lastSector = tmp;				/* set the 'next' pointer of last entry to the new created one 	*/
  lastSector = &(tmp->next);			/* set the lastSector to new 'next' pointer new created element	*/
  Anz++;
  return(tmp);
} /* end of function writeSector() */

/* function genSectors(int Op, int ImpLeft)
 * DESCRIPTION:
 * This function recusively calculates the sectors with appropriate charge, where it is ment to start with 
 * Op = (NumberOfOperators-1) ans decreases the operator-index in each step 					*/
void genSectors(int Op, int ImpLeft){
 int c,A;
 /* Abbruchbedingungen 1: jeder Operator kann maximal 1 Einheit einer speziellen Ladung tragen, falls fuer eine Ladung gilt: 
  			  (Abweichung vom Sollwert) > #(noch zu verteilenden Operatoren) = ImpLeft => dann Abbruch		*/
 for(c=0; c < NumberOfCharges; c++) if(charges[c]*charges[c] > ImpLeft*ImpLeft) return;
 if (Op == -1) { 						/* Multiplizitaet aller Operatoren gesetzt 			*/
   if (ImpLeft > 0) return;					/* Abbruchbedingungen 3: Zustand hat nicht genug impurities	*/
   append_found_Sector(Konf);					/* ansonsten: gueltigen Sector gefunden				*/
   return;
 }
 if (ImpLeft == 0) { 						/* Zustand hat richtige anzahl von Impurities			*/
   for(c=Op; 0<= c; c--) Konf[c] = 0;				/* gemaess Abbruchbed. 1 & 2 hat Zustand dann auch geforderte   */
   append_found_Sector(Konf);					/*         Ladungen => Sector gefunden				*/
   return;
 }
 for(A=0; A<= ImpLeft; A++) {					 		/* Rekursion 					*/
   Konf[Op] = A;
   for(c=0; c < NumberOfCharges; c++) charges[c] -= OpCharge[Op*4 + c]*A;	/* berechnen der Ladungen der aktuellen Konf. 	*/
   genSectors(Op-1,ImpLeft-A);							/* rekursiver Abstieg				*/
   for(c=0; c < NumberOfCharges; c++) charges[c] += OpCharge[Op*4 + c]*A;	/* restore der Werte vor rekursivem Abstieg 	*/
 }
} /* end of function genSectors() */

/* function generate_Sectors(int *Ch)
 * DESCRIPTION:
 * Input is an array of integer 'Ch' containing the values for the 4 charges: Ch = {S1,S2,J1,J2}, 
 * the values of Ch are not changed.
 * 'Impurities' defines the total number of operators the states consist of.
 * You need to call the function define_default_Operator_charges() before using this function. */
Type_Mixing_Sectors* generate_Sectors(int *Ch, int Impurities){
 int i;
 Type_Mixing_Sectors *Mixing_Sectors;
 if (Operator_Charges_defined == 0) {				/* check whether operator charges have been defined before 	*/
      printf("\nERROR: please define operator charges befor calling \"generate_Sectors\"\n\n"); exit(1); }
 charges = (int*) malloc(NumberOfCharges*sizeof(int));		/* copy given charges to seperate array 			*/
 for(i=0; i< NumberOfCharges; i++) charges[i] = Ch[i];
 Konf = (int*) malloc(sizeof(int)*NumberOfOperators);		/* create working configuration 'Konf' 				*/
 genSectors(NumberOfOperators-1, Impurities);			/* calculate all possible Sectors 				*/
 free(Konf);							/* free memory of working configuration 'Konf' 			*/
 free(charges);							/* free memory of temporary copy of given charges		*/
 Mixing_Sectors = create_list_header();
 define_Sector_Operators(Mixing_Sectors, OpOrder, NumberOfOperators);
 Mixing_Sectors->SectorList = Sectors;				/* save value of Sectors					*/
 Mixing_Sectors->NumberOfSectors = Anz;
 Sectors = NULL; Anz = 0;					/* set Sector back to initial value of NULL			*/
 return(Mixing_Sectors);
} /* end of function generate_Sectors() */

/* function compress_Sectors(Type_Mixing_Sectors *Mixing_Sectors)
 * DESCRIPTION:
 * This function removes from the sector list of 'Mixing_Sectors' all operators wich aren't needed in any of the sectors. 
 * The internal variables 'umberOfSectors', 'NumberOfOperators' and 'OperatorList' are changed to the appropriate values 	*/
void compress_Sectors(Type_Mixing_Sectors *Mixing_Sectors){
  int i,neededIndex=0, NewNumberOfOp=0;
  char **NewOperatorList;
  Type_Konf *tmp;
  int *occurrences;
  int *compressedSector;
  int *compressOccur;		/* each integer refferes to an element in list 'Sector' of Type_Konf, whose operator contibutes	*/
  				/* the elements of compressOccur have to be in incresing order 					*/
  if(Mixing_Sectors == NULL) {printf("   WARNING: structure 'Mixing_Sectors' is empty, no compression done!\n"); return; }
  if(Mixing_Sectors->OperatorList == NULL) 
      {printf("   WARNING: No operators in structure 'Mixing_Sectors', no compression done!\n"); return; }
  if(Mixing_Sectors->NumberOfSectors == 0) {printf("   WARNING: sector list empty, no compression done!\n"); return; }
  /* 'occurrences ' is array of integer, each element counts total occurrences of the corresponding operator 			*/
  occurrences = (int*) malloc(sizeof(int)*Mixing_Sectors->NumberOfOperators);		/* in all sectors together   		*/ 
  if(occurrences  == NULL) {printf("   MESSAGE: memory overflow\n"); exit(1);}
  for(i=0; i<Mixing_Sectors->NumberOfOperators; occurrences[i++]=0);			/* initialize array with zero 		*/
  tmp = Mixing_Sectors->SectorList;
  while(tmp != NULL){ 									/* fill occurrences 			*/
     for(i=0; i<Mixing_Sectors->NumberOfOperators; i++) occurrences[i] += tmp->Sector[i];
     tmp = tmp->next;  
  } /* end of while */
  for(i=0; i<Mixing_Sectors->NumberOfOperators; i++) if(occurrences[i] >0) NewNumberOfOp++;   /* count new number of Operayors	*/
  compressOccur = (int*) malloc(sizeof(int)*NewNumberOfOp);				/* generate compressOccur: copy index of*/	
  if(compressOccur  == NULL) {printf("   MESSAGE: memory overflow\n"); exit(1);}
  for(i=0; i<Mixing_Sectors->NumberOfOperators; i++) if(occurrences[i] >0) compressOccur[neededIndex++] = i; /* contibuting op  */
  tmp = Mixing_Sectors->SectorList;
  while(tmp != NULL){ 									/* genertate compressed sectors: 	*/
     compressedSector = (int*) malloc(sizeof(int)*NewNumberOfOp);			/* memory for new Sector entry		*/
     if(compressedSector  == NULL) {printf("   MESSAGE: memory overflow\n"); exit(1);}
     for(i=0; i<NewNumberOfOp; i++) compressedSector[i]=tmp->Sector[compressOccur[i]];	/* copy relevant operator exit. 	*/
     free(tmp->Sector);									/* free old sectors memory 		*/
     tmp->Sector = compressedSector;							/* set list-pointer to new sector	*/
     tmp = tmp->next;  
  } /* end of while */
  NewOperatorList = (char**) malloc(sizeof(char*)*NewNumberOfOp);			/* memory for new operator list 	*/
  if(NewOperatorList  == NULL) {printf("   MESSAGE: memory overflow\n"); exit(1);}
  for(i=0; i<NewNumberOfOp; i++) NewOperatorList[i]=Mixing_Sectors->OperatorList[compressOccur[i]]; /* keep relevant operators	*/
/*  printf("            | ");
  for(i=0; i<NewNumberOfOp; i++) printf("%s | ",NewOperatorList[i]);
  printf("\n"); */
  define_Sector_Operators(Mixing_Sectors, NewOperatorList, NewNumberOfOp);	/* write list of operators to 'Mixing_Sectors'	*/
  free(NewOperatorList);								/* free allocated memory		*/
  free(occurrences);
  free(compressOccur);
}/* end of function compress_Sectors() */

/* function write_Sectors_to_Screen(Type_Konf* List)
 * DESCRIPTION:
 * This function writes all sectors to standard output */
void write_Sectors_to_Screen(Type_Mixing_Sectors *Mixing_Sectors ){
  int i;
  Type_Konf* tmp;
  if(Mixing_Sectors == NULL) {printf("   MESSAGE: empty structure \n"); /* exit(1); */ return;}
  tmp = Mixing_Sectors->SectorList;
  if(Mixing_Sectors->SectorList == NULL) {printf("   MESSAGE: No sectors found with given charges \n"); /* exit(1); */ return;}
  printf("   MESSAGE: %d mixing sectors found: \n",Mixing_Sectors->NumberOfSectors);
  printf("            | ");
  for(i=0; i<Mixing_Sectors->NumberOfOperators; i++) printf("%s | ",Mixing_Sectors->OperatorList[i]);
  printf("\n");
  while(tmp != NULL){ 
     printf("            | ");
     for(i=0; i<Mixing_Sectors->NumberOfOperators; i++) printf("%d | ",tmp->Sector[i]);
     printf("\n");
     tmp = tmp->next;
  }
}/* end of function write_Sectors_to_Screen() */

/* function write_Sectors_to_File(Type_Konf* List)
 * DESCRIPTION:
 * This function writes all sectors to standard output */
void write_Sectors_to_File(Type_Mixing_Sectors *Mixing_Sectors , char *FileName){
  Type_Konf* tmp;
  int i;
  FILE *FilePointer;
  if(Mixing_Sectors == NULL) return;
  FilePointer = fopen(FileName,"w+");
  tmp = Mixing_Sectors->SectorList;
  if(tmp == NULL) {printf("   MESSAGE: No sectors found with given charges \n"); exit(1); }
  fprintf(FilePointer,"/*                 *****************************************************************\n");
  fprintf(FilePointer,"//                 * FILE:        %s                                      *\n",FileName);
  fprintf(FilePointer,"//                 * computer generated output file by C-programm states.c         *\n");
  fprintf(FilePointer,"//                                   on %s", today());
  fprintf(FilePointer,"//                 ****************************************************************/\n\n");
  fprintf(FilePointer,"// MESSAGE: order of Operators: \n");
  fprintf(FilePointer,"//          | ");
  for(i=0; i<Mixing_Sectors->NumberOfOperators; i++) fprintf(FilePointer,"%s | ",Mixing_Sectors->OperatorList[i]);
  fprintf(FilePointer,"\n");
  fprintf(FilePointer,"// MESSAGE: %d mixing sectors found: \n",Mixing_Sectors->NumberOfSectors);
  while(tmp != NULL){ 
     fprintf(FilePointer,"            | ");
     for(i=0; i<Mixing_Sectors->NumberOfOperators; i++) fprintf(FilePointer,"%d | ",tmp->Sector[i]);
     fprintf(FilePointer,"\n");
     tmp = tmp->next;
  }
  fprintf(FilePointer,"\n");
  fclose(FilePointer);
}/* end of function write_Sectors_to_File() */

/* function compare_Solutions(Type_Konf* List)
 * DESCRIPTION:
 * This function compares two lists of type 'Type_Konf' if they equal. If they equal value "0" is returned,
 * otherwise "1" is returned.  */
int compare_Solutions(Type_Mixing_Sectors *Ms1, Type_Mixing_Sectors *Ms2){
  Type_Konf *t1;
  Type_Konf *t2;
  int i;
  if ((Ms1 == NULL) && (Ms2 == NULL)) return(0);
  t1 = Ms1->SectorList; t2 = Ms2->SectorList;
  if((Ms1->NumberOfSectors != Ms2->NumberOfSectors) || (Ms1->NumberOfOperators != Ms2->NumberOfOperators)) return(1);
  while(t1 != NULL){
   for(i=0; i<Ms1->NumberOfOperators; i++) if(t1->Sector[i] != t2->Sector[i]) return(1);
   t1 = t1->next; t2 = t2->next;
  }
  return(0);  
} /* end of function compare_Solutions() */

/* ******************************************************* end of module  ***************************************************** */

/* ******************************************************* Testing procedures ************************************************* */
/* for testing reasons */
void test_genSectors(int Op, int ImpLeft){
 int c,A,i;
 if (Op == -1) { 						/* Multiplizitaet aller Operatoren gesetzt 			*/
   int newCharges[4] ={0,0,0,0};
   if (ImpLeft > 0) return;					/* Abbruchbedingungen 3: Zustand hat nicht genug impurities	*/
   for(i=0; i<NumberOfOperators; i++) for(c=0; c<4; c++) newCharges[c] += OpCharge[i*4+c]*Konf[i];
   for(c=0; c<4; c++) if(newCharges[c] != charges[c]) return;
   append_found_Sector(Konf);
   return;
 }
 for(A=0; A<= ImpLeft; A++) {					 /* Rekursion 							*/
   Konf[Op] = A;
   test_genSectors(Op-1,ImpLeft-A);
 }
} /* end of function genSectors() */

Type_Mixing_Sectors* test_generate_Sectors(int *Ch, int Impurities){
 int i;
 Type_Mixing_Sectors *Mixing_Sectors;
 if (Operator_Charges_defined == 0) {				/* check whether operator charges have been defined before 	*/
      printf("\nERROR: please define operator charges befor calling \"generate_Sectors\"\n\n"); exit(1); }
 charges = (int*) malloc(NumberOfCharges*sizeof(int));		/* copy given charges to seperate array 			*/
 for(i=0; i< NumberOfCharges; i++) charges[i] = Ch[i];
 Konf = (int*) malloc(sizeof(int)*NumberOfOperators);		/* create working configuration 'Konf' 				*/
 test_genSectors(NumberOfOperators-1, Impurities);		/* calculate all possible Sectors 				*/
 free(Konf);
 free(charges);							/* free memory of temporary copy of given charges		*/
 Mixing_Sectors = create_list_header();
 define_Sector_Operators(Mixing_Sectors, OpOrder, NumberOfOperators);
 Mixing_Sectors->SectorList = Sectors;				/* save value of Sectors					*/
 Mixing_Sectors->NumberOfSectors = Anz;
 Sectors = NULL; Anz = 0;					/* set Sector back to initial value of NULL			*/
 return(Mixing_Sectors);
} /* end of function test_generate_Sectors() */

/* for testing reasons */
void compare2(Type_Mixing_Sectors *Ms1, Type_Mixing_Sectors *Ms2){
  Type_Konf *t1;
  Type_Konf *t2;
  int i;
  int line=1;
  if ((Ms1 == NULL) && (Ms2 == NULL)) { printf(" empty structure  \n"); return;}
  t1 = Ms1->SectorList; t2 = Ms2->SectorList;
  if (Ms1->NumberOfOperators != Ms2->NumberOfOperators)
    {printf("\n   ERROR: mismatch between NumberOfOperators (%d,%d)!\n\n",Ms1->NumberOfOperators,Ms2->NumberOfOperators); exit(1);}
  if (Ms1->NumberOfSectors != Ms2->NumberOfSectors)
    {printf("\n   ERROR: mismatch between NumberOfSectors (%d,%d)!\n\n",Ms1->NumberOfSectors,Ms2->NumberOfSectors); exit(1);} 
  while(t1 != NULL){
   for(i=0; i<Ms1->NumberOfOperators; i++) if(t1->Sector[i] != t2->Sector[i]) {
	printf("\n   ERROR: mismatch found in line %d !\n\n",line);
	write_Sectors_to_Screen(Ms1); 
	write_Sectors_to_Screen(Ms2); 
	exit(1);
   } /* end of if */
   line++;
   t1 = t1->next; t2 = t2->next;
  }
  printf(" agreement found (%d sectors with given charges) \n",Ms1->NumberOfSectors);
} /* end of function  compare2() */

/* for testing reasons */
void test_ALL_Charges(int Imp){
 int S1,S2,J1,J2;
 int maximp;
 int newCharges[4];
 if (Operator_Charges_defined == 0) {				/* check whether operator charges have been defined before 	*/
      printf("\nERROR: please define operator charges befor calling \"generate_Sectors\"\n\n"); exit(1); }
 maximp = 2*Imp;
 for(S1 = -maximp; S1<= maximp; S1++)
   for(S2 = -maximp; S2<= maximp; S2++)
      for(J1 = -maximp; J1<= maximp; J1++)
         for(J2 = -maximp; J2<= maximp; J2++) {
	    newCharges[0] = S1; newCharges[1] = S2;
	    newCharges[2] = J1; newCharges[3] = J2;
            printf("   Comparing sectors with charge {%d,%d,%d,%d}: ",S1,S2,J1,J2);
            compare2(generate_Sectors(newCharges, Imp), test_generate_Sectors(newCharges, Imp));
	 } 
} /* end of function  test_ALL_Charges*/

/* ************************************************** end of testing procedures *********************************************** */
