/*                                               *****************************************************************
//                                               * FILE:        generateSectors.h                                *
//                                               * written by:  Alexander Hentschel                              *
//                                               *              Humboldt University Berlin                       *
//                                               *              Alexander.Hentschel@physik.hu-berlin.de          *
//                                               *              November 2006                                    *
//                                               ****************************************************************/

/* COMMENT on inner working of this module:
 * the charges are all multiplied by 2 to obtain a representation, dealing only with integer numbers */


#ifndef GENERATE_SECTORS_H
#define GENERATE_SECTORS_H


/* definition Type_Konf
 * DESCRIPTION:
 * elements of a dynamic list, each element containing a possible sector 
 * Sector is ment to point on an array where each element definies how often the corresponding operator 
 * appears in the state */
typedef struct Konf_Element {
  int *Sector;
  struct Konf_Element *next;
} Type_Konf;

/* definition Type_Konf
 * DESCRIPTION:
 * elements of a dynamic list, each element containing a possible sector 
 * Sector is ment to point on an array where each element definies how often the corresponding operator 
 * appears in the state */
typedef struct {
  int NumberOfSectors;
  int NumberOfOperators;
  char** OperatorList;
  Type_Konf *SectorList;
} Type_Mixing_Sectors;

/* function create_list_header()
 * DESCRIPTION:
 * This function creates a list header of type Type_Mixing_Sectors and returns the corresponding pointer
 * The OperatorList and SectorList pointer are both set to zero. */
extern Type_Mixing_Sectors* create_list_header();

/* function free_Sector_List(Type_Mixing_Sectors *Mixing_Sectors)
 * DESCRIPTION:
 * frees memory of Sector list, created by generate_Sectors() or append_found_Sector() 
 * return value is NULL */
extern Type_Mixing_Sectors* free_Sector_List(Type_Mixing_Sectors *Mixing_Sectors);

/* function generate_Sectors(int *Ch, int Impurities)
 * DESCRIPTION:
 * Input is an array of integer 'Ch' containing the values for the 4 charges: Ch = {S1,S2,J1,J2}, 
 * the values of Ch are not changed.
 * 'Impurities' defines the total number of operators the states consist of.
 * You need to call the function define_default_Operator_charges() before using this function. */
extern Type_Mixing_Sectors* generate_Sectors(int *Ch, int Impurities);

/* function define_default_Operator_charges()
 * DESCRIPTION: 
 * defines charges of every operator, where Operators are normal ordered    
 * normal ordering als well as charges are in convention of the paper hep-th/0603008 */
extern void define_default_Operator_charges();

/* function compress_Sectors(Type_Mixing_Sectors *Mixing_Sectors)
 * DESCRIPTION:
 * This function removes from the sector list of 'Mixing_Sectors' all operators wich aren't needed in any of the sectors. 
 * The internal variables 'umberOfSectors', 'NumberOfOperators' and 'OperatorList' are changed to the appropriate values 	*/
extern void compress_Sectors(Type_Mixing_Sectors *Mixing_Sectors);

/* function write_Sectors_to_Screen(Type_Mixing_Sectors *Mixing_Sectors )
 * DESCRIPTION:
 * This function writes all sectors to standard output */
extern void write_Sectors_to_Screen(Type_Mixing_Sectors *Mixing_Sectors );

/* function write_Sectors_to_File(Type_Mixing_Sectors *Mixing_Sectors , char *FileName)
 * DESCRIPTION:
 * This function writes all sectors to standard output */
extern void write_Sectors_to_File(Type_Mixing_Sectors *Mixing_Sectors , char *FileName);

/* function compare_Solutions(Type_Mixing_Sectors *Ms1, Type_Mixing_Sectors *Ms2)
 * DESCRIPTION:
 * This function compares two lists of type 'Type_Konf' if they equal. If they equal value "1" is returned,
 * otherwise "0" is returned.  */
extern int compare_Solutions(Type_Mixing_Sectors *Ms1, Type_Mixing_Sectors *Ms2);

/* function test_ALL_Charges(int Imp)
 * DESCRIPTION:
 * This function generates all possible charges, which a state with given impurity 'Imp' can carry.
 * The result of function generate_Sectors() is counterchecked against a rusult of an other more general (and much slower) 
 * function. If a disagreement arises, an error massage is printed and the calculation is aborted. */
extern void test_ALL_Charges(int Imp);

#endif
