/*                                               *****************************************************************
//                                               * FILE:        generate_states.c                                *
//                                               * written by:  Alexander Hentschel                              *
//                                               *              Humboldt University Berlin                       *
//                                               *              Alexander.Hentschel@physik.hu-berlin.de          *
//                                               *              July 2006                                        *
//                                               ****************************************************************/

#include "define_states.def"

#ifndef COMPILE_CPP
  #define _GNU_SOURCE
  #include <stdlib.h>
  #include <stdio.h>
  #include <limits.h>
  #include <string.h>
  #include <ctype.h>
  #define STREAM_TYPE FILE 
#endif

#ifdef COMPILE_CPP
  #include <iostream>  	/* wegen cout cin 		*/
  #include <fstream> 	/* wegen Dateistreamobjekt	*/
  #include <string> 	/* wegen Datentyp string	*/
  #define STREAM_TYPE std::ifstream
#endif

#include "generate_states.h"
#include "recycleInput.h"
#include "define_states.def"

static int NumOp;     /* Number of Operators */
static int NumFermOp; /* Number of Fermionic Operators */
static int NumMo;     /* Number of Modes */
static int NumEx=0;   /* Total number of all excitations */
static int AgZ=0;     /* Total number of generated states */
static FILE *Output;  /* output File */
static int *Block;    /* Array defining Subblock of Hamiltonian matrix: defifines how many Exciations each Operator flavour carries */
static int *State;    /* defines state: Array for each Operator and each Mode: exciation number */

static int function_getSector_active = 0; /* 0 = no, 1=yes */

/* function numberOfStatesForOneMode()
 * return value: number of possibilities to distribute excitations on one mode over the operators */
int numberOfStatesForOneMode(int excitations, int NumberOfOperators, int FermionicOperators){
  int temp=1; int i,t;
  if ((NumberOfOperators == FermionicOperators) && (excitations > NumberOfOperators)) /* only fermionic operators */
     return(0);
  if (NumberOfOperators==1) return(1);
  if (excitations==1) return(NumberOfOperators);
  if (FermionicOperators >0)
     return(   numberOfStatesForOneMode(excitations,NumberOfOperators-1,FermionicOperators-1)
             + numberOfStatesForOneMode(excitations-1,NumberOfOperators-1,FermionicOperators-1));
  for(i=0; i< excitations; i++) { 
      t = numberOfStatesForOneMode(excitations-i,NumberOfOperators-1,FermionicOperators-1);
      if(temp >= (INT_MAX-t)) {
           #ifndef AllowMoreThan2E9States
             printf("\nERROR: number of states is passing upper bound of %d states! \n\n",INT_MAX);
	     exit(1);
	   #endif
           #ifdef AllowMoreThan2E9States
             printf("\n   WARNING: number of states is passing upper bound of %d states! \n\n",INT_MAX);
	   #endif
      }
      temp += t;
  }
  return(temp);
}/* end of function numberOfStatesForOneMode */

/* function numberOfStates()
 * return value: total number of states needed to be generated to calculate the whole Hamiltonian matrix   */
int calculateNumberOfStates(int ExcNumbers[], int NumberOfOperators, int FermionicOperators, int NumberOfModes){
  int i,t;
  int needed_states=1;
  for (i=0; i< NumberOfModes; i++) {
      t = numberOfStatesForOneMode(ExcNumbers[i],NumberOfOperators,FermionicOperators);
      if(needed_states >= ((int) (INT_MAX/t))) {
           #ifndef AllowMoreThan2E9States
             printf("\nERROR: number of states is passing upper bound of %d states! \n\n",INT_MAX);
	     exit(1);
	   #endif
           #ifdef AllowMoreThan2E9States
             printf("\n   WARNING: number of states is passing upper bound of %d states! \n\n",INT_MAX);
	   #endif
      }
      needed_states *= t;
  }
  return(needed_states);
}/* end of function numberOfStates */


/* rekursive Funktion generate_Block_States
 * - durchlaeuft in jeder Instanz eine (der verschiedenen) Moden eines der Operatoren
 * - Anregungszahl der Mode erhaelt Werte 0,...,ExNumber, restliche Anregungen werden zur weiteren Verteilung
 *   in naechsten Operatorsektor verschoben (nextsubstate)
 * - vor Backtracking wird unspruenglicher Anregungswert im aktuellen Sektor (ExNumber) restored 
 *   und Anregungswert des naechsten Operatorsektors wieder auf null gesetzt
 * Input:	Mode 		- Pointer in Array State auf aktuell zu bearbeitende mode
 *		neededEx	- Anzahl Anregungen, die noch auf aktuellen Operator zu verteilen sind 
 *		ExLeft		- Anzahl aller Anregungen im aktuellen Subsector, die noch verteilt werden koennen 
 *		MNextSec	- Anzahl Moden, die in den naechsten Zustandsteil verschoben wurden
 *		ModesLeft	- Anzahl noch nicht abgearbeiteter Moden in aktuellem Operatorsektor (ModesLeft = NumMo,...,1)
 *				  aktuelle Mode wird in ModesLeft mitgezaehlt
 *		OpLeft		- Anzahl noch nicht abgearbeiteter Operatoren (OpLeft = NumOp,...,1)
 *				  aktueller Operatorsektor wird in OpLeft mitgezaehlt
 */
void generate_Block_States(int* Mode, int neededEx, int ExLeft, int MNextSec, int ModesLeft, int OpLeft){
  int *nextsubstate;   /* Pointer in Array State auf Teil des als naechsten aktuell zu bearbeitenden Operators                   */
  int ExNumber;        /* Anzahl der Anregungen der entsprechenden Mode dieser Rekursionsinstanz                                 */
  int MaxEx = 1;       /* Anzahl Anregungen (der aktuellen Mode) die (aktueller) Operator tragen kann - default =1 (fermionisch) */
  int i;
  nextsubstate = Mode + NumMo;
  ExNumber = *Mode; /* aktuelle Mode */
  
  /* Abbruchbedingungen 0: nur fermionische Operatoren */	/* jeder der OpLeft fermion. Op. kann maximal eine Anregungen  */
  if ((NumOp == NumFermOp) && (OpLeft < ExNumber)) return; 	/* der aktuellen Mode tragen - sonst verschwindet Zustand (=0) */

  /* Abbruchbedingungen 1: nicht mehr genuegend Anregungen uebrig im aktuellen Subsector */
  if(neededEx > ExLeft) return; /* kein zulaessiger Zustand mehr generierbar */

  /* Abbruchbedingungen 2: letztes Operatorsegment erreicht */
  if(OpLeft == 1) { /* aktueller Operatorsektor wird in OpLeft mitgezaehlt */
     if (NumOp == NumFermOp) /* ueberpruefen des letzten Sectors auf Mehrfach-Anregung fermionischer Moden */
       for(i=0; i< NumMo; i++) if (Mode[i]> 1) return; /* falls nur fermionische Operatoren beteiligt sind */
     AgZ++;
     writeStateToFile(State,NumOp,NumMo,Output);
     return;
  } /* Ende Abbruchbed. 2 */
  
  /* Abbruchbedingungen 3: Subsector hat geforderte Anzahl an Anregungen (es gilt nun: nicht letztes Operatorsegment)*/
  if(neededEx == 0) { 
     if (ModesLeft > 0) { /* und noch Moden unbetrachtet */
        for(i = 0; i < ModesLeft; i++){ /* verschiebe verbleibende Moden-Anregungen in naechsten Operator-Subsector*/
           nextsubstate[i]=Mode[i];
	   ExLeft -= Mode[i]; /* Vermutung: ueberfluessig !!!! */
           Mode[i]=0;	
	   MNextSec += nextsubstate[i];
        }
     }
     /* rekursiver Abstieg in naechsten Operator-subsector*/
     generate_Block_States(State + (NumOp-OpLeft+1)*NumMo,Block[NumOp-OpLeft+1],MNextSec,0,NumMo,OpLeft-1);
     if (ModesLeft > 0) {      /* restore unspruenglichen Anregungswert (ExNumber) */
        for(i = 0; i < ModesLeft; i++){ /* verschiebe verbleibende Moden-Anregungen in naechsten Operator-Subsector*/
           Mode[i]=nextsubstate[i];
	   nextsubstate[i]=0;
        }
     }
     return;
  } /* Ende Abbruchbed. 3 */
  /* es gilt nun: (nicht letztes Operatorsegment) && (ExLeft >= neededEx) && (ModesLeft > 0)			*/ 
  /* Rekursiver Abstieg */			   	/* Index des aktuellen Operators : (NumOp-OpLeft)  	*/
  MaxEx = (ExNumber < neededEx)? ExNumber : neededEx;	/* maximale Anregung = min(ExNumber,neededEx) 		*/
  if ((NumOp-OpLeft)<NumFermOp) 			/* falls aktueller Operator fermionisch ist		*/
       MaxEx = (MaxEx<1)? MaxEx : 1;   			/* maximal moegliche Anregung, min(MaxEx,1)		*/
  /* rekursiver Abstieg: auf naechste Mode des gleichen Operators nur falls (neededEx > 0) und (ModesLeft > 0)	*/
  for(i=MaxEx; i>=0; i--) {
    *Mode = i;
    *nextsubstate = ExNumber -i;
    generate_Block_States(Mode+1, neededEx-i, ExLeft-ExNumber, MNextSec+ExNumber-i, ModesLeft-1, OpLeft);
    *Mode = ExNumber;     /* restore unspruenglicher Anregungswert (ExNumber) */
    *nextsubstate = 0;
  } /* end of for(i=neededEx; i>=0; i--) */
  return;	   
} /* end of function generate_Block_States */

int* getSector(){
  if(function_getSector_active == 0){ 
    printf("\nERROR: please call function \"getSector()\" only during states generation process !\n\n");
    exit(1);
  }
  return(Block);
}

/* rekursive Funktion gen_all_Blocks
 * Input: 	vo - Anzahl der verbleibenden Operatoren auf die Moden verteilt werden muessen (aktueller Operator zaehlt mit)
 *		ve - Anzahl (aller) noch zu verteilenden Anregungen 
 * Aufgabe:	belegt Array Block mit Modenzahlen  */
void gen_all_Blocks(int vo, int ve){ 
  int i, MaxEx;
  /* Abbruchbedingungen 0: nur fermionische Operatoren */	/* vo fermionische Op. koennen je NumMo Anregungen tragen */
  if ((NumOp == NumFermOp) && ((vo*NumMo) < ve)) return; 	/* falls ve > vo*NumMo: keine zugelassenen Sectoren generierbar */
  
  /* Abbruchbedingungen 1: keine Anregungen mehr zu verteilen*/
  if(ve == 0){ /* keine Anregungen mehr zu verteilen */
     for(i=NumOp-vo; i<NumOp; i++) Block[i] = 0;
     writeSector(Block,NumOp,Output);
     generate_Block_States(State, Block[0], NumEx, 0, NumMo, NumOp);
     return;
  } /* Ende Abbruchbedingung 1 */
  
  /* Abbruchbedingungen 2: nur ein Operator uebrig */
  if(vo == 1){ /* nur ein Operator uebrig */
     Block[NumOp-1] = ve;
     writeSector(Block,NumOp,Output);
     generate_Block_States(State, Block[0], NumEx, 0, NumMo, NumOp);
     return;
  } /* Ende Abbruchbedingung 2 */
  
  /* rekursiver Abstieg */        /* Index des aktuellen Operators:  (NumOp-vo) */
  if ((NumOp-vo) < NumFermOp) MaxEx = (ve<NumMo)? ve : NumMo;   /* aktueller Op. ferm.: kann maximal NumMo Anregungen tragen  */
    else MaxEx = ve;						/* aktueller Op. bos.:  kann beliebig viele Anregungen tragen */
  for(i=MaxEx; i>=0; i--) { 	  
           Block[NumOp-vo] = i;
           gen_all_Blocks(vo-1,ve-i);
  } /* End of for(i=MaxEx; i>=0; i--) */
} /* end of function gen_all_Blocks  */

int gen_all_States(int *ExcNumbers, int NumberOfOperators, int FermionicOperators, int NumberOfModes, FILE *OutputFile){
  int i;
  function_getSector_active = 1; /* setting ability to call function getSector() */
  NumOp = NumberOfOperators;
  NumMo = NumberOfModes;
  NumFermOp = FermionicOperators;
  AgZ = NumEx = 0;
  if(NumOp==0) {
    printf("\nERROR: No Operators given to distribute states on!\n");
    printf("       in function gen_all_States \n\n");
    exit(1);
  }
  if(NumMo==0) {
    printf("\nERROR: No Modes given to distribute!\n");
    printf("       in function gen_all_States \n\n");
    exit(1);
  }
  if (NumberOfOperators < FermionicOperators) {
    printf("\nERROR: Number of fermionic operators is greater \n");
    printf("       than total number of all operators!\n");
    printf("       Error in function gen_all_States \n\n");
    exit(1);
  }
  for(i=0; i<NumMo; i++) NumEx += ExcNumbers[i];
  State = (int*) calloc(NumOp*NumMo,sizeof(int));
  Block = (int*) malloc(sizeof(int)*NumOp);
  Output = OutputFile;
  for(i=0; i<NumMo; i++) State[i]=ExcNumbers[i]; /* write given excitations on first operator */
  gen_all_Blocks(NumOp,NumEx);
  free(State);
  free(Block);
  function_getSector_active = 0; /* unsetting ability to call function getSector() */
  return(AgZ);
} /* end of function gen_all_States */

/* function check_consistency_of_Sector_and_ExcNumbers
 * DESCRIPTION:
 * This function provides several consistency tests of the excitation Numbers 'ExcNumbers' and the geiven sector 'Sector'.
 * It is recommended to call this function before using gen_States_of_One_Sector() when dealing with entered sectors 		*/
void check_consistency_of_Sector_and_ExcNumbers(int *ExcNumbers, int *Sector, int NumberOfOperators, int NumberOfModes){
  int i;
  int temp=0;
  function_getSector_active = 1; /* setting ability to call function getSector() */
  AgZ = NumEx = 0;
  if (Sector == NULL) {  printf("\nERROR: no \"HamiltonianSector\" specified !\n\n");  exit(1); }
  if(NumberOfOperators==0) {
    printf("\nERROR: No Operators given to distribute states on!\n");
    printf("       in function gen_all_States \n\n");
    exit(1);
  }
  if(NumberOfModes==0) {
    printf("\nERROR: No Modes given to distribute!\n");
    printf("       in function gen_all_States \n\n");
    exit(1);
  }
  for(i=0; i<NumberOfModes; i++) NumEx += ExcNumbers[i];
  for(i=0; i<NumberOfOperators; i++) { /* Testen des angegebenen Blocks (unter Pointer Sector) */
     if(Sector[i] < 0) { /* negative Nummer einer Moden-Anregungen */
        printf("\nERROR: negative number of mode excitation given in \"HamiltonianSector\" (number %d) !\n",i+1);
        printf("       in function gen_States_of_One_Sector \n\n");
        exit(1);
     }
     if(Sector[i] > NumEx) { /* zu grosse Nummer einer Moden-Anregungen (groesser als Gesamtzahl aller Anregungs-Zahlen) */
        printf("\nERROR: mode excitation number %d of \"HamiltonianSector\" is greater\n",i+1); 
        printf("       than sum of all exciations specified in ExcNumbers !\n");
        printf("       in function gen_States_of_One_Sector \n\n");
        exit(1);
     }
     temp += Sector[i];
  }
  if(temp != NumEx) {
     printf("\nERROR: total sum of excitations given in \"HamiltonianSector\" differs\n"); 
     printf("       from those given in \"ExcNumbers\" \n");
     printf("       in function gen_States_of_One_Sector \n\n");
     exit(1);
  }
  if(NumberOfModes==0) {
    printf("\nERROR: No Modes given to distribute!\n");
    printf("       in function gen_all_States \n\n");
    exit(1);
  }
} /* gen_States_of_One_Block */

/* function gen_States_of_One_Sector
 * return value:   number of generated states 
 * DESCRIPTION:
 * generates all possible states of the Hamiltonian subsector specified by the array Sector  
 * calls function writeStateToFile() every time a state is generated */
int gen_States_of_One_Sector(int *ExcNumbers, int *Sector, int NumberOfOperators, int FermionicOperators, int NumberOfModes, FILE *OutputFile){
  int i;
  function_getSector_active = 1; /* setting ability to call function getSector() */
  NumOp = NumberOfOperators;
  NumMo = NumberOfModes;
  NumFermOp = FermionicOperators;
  AgZ = NumEx = 0;
  for(i=0; i<NumMo; i++) NumEx += ExcNumbers[i];
  State = (int*) calloc(NumOp*NumMo,sizeof(int));
  Block = Sector;
  Output = OutputFile;
  for(i=0; i<NumMo; i++) State[i]=ExcNumbers[i];
  generate_Block_States(State, Block[0], NumEx, 0, NumMo, NumOp);
  free(State);
  function_getSector_active = 0; /* setting ability not to call function getSector() */
  return(AgZ);
} /* gen_States_of_One_Block */

void plotn(char* Zeichenkette, int number, FILE *Output){
  int i;
  if (number < 0) return;
  for(i=0; i<number; i++) {
    if ( strncmp(&(Zeichenkette[i]),"\0", 1) == 0) return;
    fprintf(Output,"%c",Zeichenkette[i]);
  }
}

void plotUpToNewline(char* Zeichenkette, int number, FILE *Output){
  int i;
  if (number <= 0) return;
  for(i=0; i<number-1; i++) {
     if (Zeichenkette[i] == '\n') return;
     fprintf(Output,"%c",Zeichenkette[i]);
  }
}

int findFirstNumberInString(char* Zeichenkette){
  int i ;
  if (Zeichenkette == NULL) return(-1);
  for(i=0; ; i++) {
    if ( strncmp(&(Zeichenkette[i]),"\0", 1) == 0) return(-1);
    if (isdigit(Zeichenkette[i]) != 0) return(i);
  }
}

int get_one_line(char **lineptr, size_t *n, STREAM_TYPE *stream){
  #ifdef COMPILE_CPP
     int len;
     if ((*stream).eof()) return(-1);
	std::string buffer; /* Buffer */
	getline((*stream), buffer);
        len = (int) buffer.length();
	*lineptr = (char*) realloc(*lineptr,2+len*sizeof(char));
	*n = len+2;
	buffer.copy(*lineptr,len);
	(*lineptr)[len] = '\n';
	(*lineptr)[len+1] = '\0';
     return(len);
     return(len);
  #else
     return(getline(lineptr,n,stream));
  #endif
}

STREAM_TYPE* C_CPP_OPEN(const char *Name, const char *modus){
  #ifdef COMPILE_CPP
     return(new STREAM_TYPE(Name));
  #else
     return(fopen(Name,modus));
  #endif
}
void C_CPP_CLOSE(STREAM_TYPE* file){
  #ifdef COMPILE_CPP
     (*file).close();;
  #else
     fclose(file);
  #endif
}

int generate_Matrix_calculation(char* InputFileName, char* OutputFilename) {
  FILE *WriteToFile;
  STREAM_TYPE *In1;
  STREAM_TYPE *In2;
  int bytes_read1 = 0;
  int bytes_read2 = 0;
  int j=1,i=1;
  char* line1 = NULL;
  char* line2 = NULL;
  In1 = C_CPP_OPEN(InputFileName,"r");
  WriteToFile = fopen(OutputFilename,"w+");
  printf("   MESSAGE: preparing FORM matrix file \"%s\".\n",OutputFilename);
  fprintf(WriteToFile,"@*                                              *****************************************************************\n");
  fprintf(WriteToFile,"@*                                              * FILE:        %s                          *\n",OutputFilename);
  fprintf(WriteToFile,"@*                                              * computer generated output file by C-programm states           *\n");
  fprintf(WriteToFile,"@*                                                               on %s", today());
  fprintf(WriteToFile,"@*                                              *****************************************************************\n\n");
  while (get_one_line(&line1, (size_t*) &bytes_read1, In1) != -1) {
     if((line1[0] != '@') && (line1[0] != '\n')){
     In2 = C_CPP_OPEN(InputFileName,"r");
       fprintf(WriteToFile,"@ ~~~~~~~~~~~~~~~~~ Hamiltonian line  %d ~~~~~~~~~~~~~~~~~ \n",i);
       fprintf(WriteToFile,"       #call %s \n",NewHamiltonianLineProc);
       fprintf(WriteToFile,"       Local BraVector = ");
       #ifdef BraVectorFlag
         fprintf(WriteToFile,"%s*",BraVectorFlag);
       #endif
       plotUpToNewline(line1, bytes_read1/sizeof(char)-1, WriteToFile);
       fprintf(WriteToFile,";\n");
       fprintf(WriteToFile,"       #call %s \n",RecycleBraVector);
       while (get_one_line(&line2, (size_t*) &bytes_read2, In2) != -1) {
         if((line2[0] != '@') && (line2[0] != '\n')) {
           fprintf(WriteToFile,"@   ~~ Hamiltonian element (%d,%d) \n",i,j++);
           fprintf(WriteToFile,"       Local KetVector = ");
           plotUpToNewline(line2, bytes_read2/sizeof(char)-1, WriteToFile);
           fprintf(WriteToFile,";\n");
           fprintf(WriteToFile,"       #call %s \n",MatrixCalculationProcedure);
	 } /* end of if*/
       } /* end of while */
       C_CPP_CLOSE(In2);
       i++; j=1;
    } /* end of if */   
  } /* end of while */
  fprintf(WriteToFile,"\n\n@ end of file %s\n",OutputFilename);
  C_CPP_CLOSE(In1);
  fclose(WriteToFile);
  free(line1); free(line2); 
  return(1);
} /* end of generate_Matrix_calculation */


void createMathematicaFile(char* ModeNames[], int* ExcNumbers, int NumberOfModes, char* FileName) {
  FILE *FilePointer;  
  int i;
  FilePointer = fopen(FileName,"w+");
  fprintf(FilePointer,"(*                                 *******************************************************************************\n");
  fprintf(FilePointer," *                                 * FILE:        %s                                                 *\n",FileName);
  fprintf(FilePointer," *                                 * computer generated output file by FORM-programm calculateMatrixElement.prc  *\n");
  fprintf(FilePointer," *                                 * Content-type: application/mathematica                                       *\n");
  fprintf(FilePointer," *                                                      %s", today());
  fprintf(FilePointer," *                                 ******************************************************************************)\n\n");
  fprintf(FilePointer,"Notebook[{\n");
  fprintf(FilePointer,"Cell[BoxData[{\n");
  fprintf(FilePointer,"    RowBox[{\\(Off[General::spell1]\\),");
  fprintf(FilePointer,"\"\\[IndentingNewLine]\"}], \"\\[IndentingNewLine]\",\n ");
  fprintf(FilePointer,"    RowBox[{\\( ");    
  for(i=0; i < NumberOfModes; i++ ) fprintf(FilePointer,"  \\(%s\\_%d = %d\\)\\(;\\)\\(\\ \\)",DefaultExcitationName, i+1,ExcNumbers[i]);    
  fprintf(FilePointer,"\\),\n        \"\\[IndentingNewLine]\"}], \"\\[IndentingNewLine]\",\n");  
  #ifdef DefaultValueLamdatilde
    fprintf(FilePointer,"    RowBox[{\\(Lamdatilde\\  = %s\\ ;\\), \" \"}], \"\\[IndentingNewLine]\",\n",DefaultValueLamdatilde);
  #endif
  #ifdef DefaultValueKappa
    fprintf(FilePointer,"    RowBox[{\\(Kappa\\  = %s\\ ;\\), \" \"}], \"\\[IndentingNewLine]\",\n",DefaultValueKappa);
  #endif
  #ifdef DefaultValuePPlus
    fprintf(FilePointer,"    RowBox[{\\(PPlus\\  = %s\\ ;\\), \" \"}], \"\\[IndentingNewLine]\",\n",DefaultValuePPlus);
  #endif
  fprintf(FilePointer,"    \"\\[IndentingNewLine]\",\n");
  
  for(i=0; i < (NumberOfModes-1); i++ ) 
      fprintf(FilePointer,"     \\(%s\\_%d = \\ \\(%s = 1\\);\\), \"\\[IndentingNewLine]\",\n",DefaultModeName,i+1,ModeNames[i]);
  fprintf(FilePointer,"     \\(%s\\_%d = \\ \\(%s = -1\\);\\), \"\\[IndentingNewLine]\",\n",DefaultModeName,i+1,ModeNames[NumberOfModes-1]);
/*  #endif
  #ifndef ModeIndices
    for(i=0; i < (NumberOfModes-1); i++ ) 
       fprintf(FilePointer,"       \\(\\(%s\\_%d = \\  1;\\)\\), \"\\[IndentingNewLine]\",\n",DefaultModeName,i+1);
    fprintf(FilePointer,"       \\(\\(%s\\_%d = \\  -1;\\)\\), \"\\[IndentingNewLine]\",\n",DefaultModeName,NumberOfModes);
  #endif */
  fprintf(FilePointer,"    \"\\[IndentingNewLine]\",\n");

  fprintf(FilePointer,"    RowBox[{\\(\\[Sum]\\+\\(Mode = 1\\)\\%c",37);   /* ascii(37) = "%"*/
  fprintf(FilePointer,"%d %s\\_Mode*%s\\_Mode\\),\n",NumberOfModes,DefaultExcitationName,DefaultModeName);
  fprintf(FilePointer,"      \"\\[IndentingNewLine]\"}], \"\\[IndentingNewLine]\", \n");
  fprintf(FilePointer,"    RowBox[{\\(Omega[n_] := \\@\\(1 + Lamdatilde*n\\^2\\);\\), \n");  
  fprintf(FilePointer,"      \" \"}], \"\\[IndentingNewLine]\", \n");  
      
  fprintf(FilePointer,"    RowBox[{\\(fFunction[n_]\\  := \\ \\@\\(\\(1\\/2\\) \\((1 + 1\\/Omega[n])\\)\\);\\),\n");
  fprintf(FilePointer,"       \" \"}], \"\\[IndentingNewLine]\",\n");
  fprintf(FilePointer,"    RowBox[{\\(gFunction[n_]\\  := \\ \\(Kappa*\\@Lamdatilde*n\\)\\/\\(1 + Omega[n]\\)*fFunction[n];\\),\n");  
  fprintf(FilePointer,"      \" \"}], \"\\[IndentingNewLine]\",\n");  	     
  fprintf(FilePointer,"      RowBox[{\" \", \\(KomplexI\\  = \\ \\[ImaginaryI];\\), \"\\[IndentingNewLine]\"}], \"\\[IndentingNewLine]\", \n");
  fclose(FilePointer); 
} /* end of function createMathematicaFile */




/* End of module generate_states*/ 


