/*                                               *****************************************************************
//                                               * FILE:        generate_states.h                                *
//                                               * written by:  Alexander Hentschel                              *
//                                               *              Humboldt University Berlin                       *
//                                               *              Alexander.Hentschel@physik.hu-berlin.de          *
//                                               *              July 2006                                        *
//                                               ****************************************************************/

#ifndef GENERATE_STATES_H
#define GENERATE_STATES_H

/* used function variables: 	
 *	NumberOfOperators  - Numbers of Operators
 *	FermionicOperators - Number of fermionic Operators (has to be <= NumberOfOperators)
 *	NumberOfModes 	   - Numbers of Modes
 *	OutputFile	   - File-pointer to output file for the generated states 
 *	ExcNumbers	   - Array of integer containing the excitation-number for each mode
 *	Sector		   - Array of integer defining a subblock of the Hamiltonian matrix
 *			     each element defines the number of impurities distributed to the corresponding operator
 * caution:
 *	NumberOfModes has to be equal to the number of elements in ExcNumber
 *	NumberOfOperators has to be equal to the number of elements in Sector
 *
 * function writeStateToFile() as specified below has to be provided by the user
 * function writeSector() as specified below has to be provided by the user
 */

/* Name of FORM procedure whitch is called every time before e new line in the Hamiltonian is started */
#define NewHamiltonianLineProc "BeginNewHamiltonianLine"

/* Name of FORM procedure calculating the matrix element */
#define RecycleBraVector "renameBra"

/* Name of FORM procedure calculating the matrix element */
#define MatrixCalculationProcedure "CalculateElement"

/* This Flag is multiplied to the bra vector  
 * if no flag is needed, please comment it out */
#define BraVectorFlag "BraFlag"
 
#define DefaultValueLamdatilde "0.1"
#define DefaultValueKappa "1"
#define DefaultValuePPlus "100"

/* function numberOfStates()
 * return value: total number of states needed to be generated to calculate the whole Hamiltonian matrix   */
extern int calculateNumberOfStates(int ExcNumbers[], int NumberOfOperators, int FermionicOperators, int NumberOfModes);

/* function check_consistency_of_Sector_and_ExcNumbers
 * DESCRIPTION:
 * This function provides several consistency tests of the excitation Numbers 'ExcNumbers' and the geiven sector 'Sector'.
 * It is recommended to call this function before using gen_States_of_One_Sector() when dealing with entered sectors 		*/
extern void check_consistency_of_Sector_and_ExcNumbers(int *ExcNumbers, int *Sector, int NumberOfOperators, int NumberOfModes);

/* function gen_States_of_One_Sector
 * return value:   number of generated states 
 * DESCRIPTION:
 * generates all possible states of the Hamiltonian subsector specified by the array Sector  
 * calls function writeStateToFile() every time a state is generated 
 * where the according arguments are passed to the function */
extern int gen_States_of_One_Sector(int *ExcNumbers, int *Sector, int NumberOfOperators,int FermionicOperators, int NumberOfModes, FILE *OutputFile);
 
/* function gen_all_States
 * return value:   number of generated states 
 * DESCRIPTION:
 * generates all possible states sector by sector and calls function writeStateToFile() every time a state is generated 
 * where the according arguments are passed to the function 
 * each time before states of a new Hamiltonian sector are generated function writeSector() is called 
 * where the according arguments are passed to the function */
extern int gen_all_States(int *ExcNumbers, int NumberOfOperators, int FermionicOperators, int NumberOfModes, FILE *OutputFile);


/* specification of function writeStateToFile
 * this function has to be implemented by the user of this library */
extern void writeStateToFile(int* State, int NumberOfOperators, int NumberOfModes, FILE *OutputFile);

/* specification of function writeSector
 * this function has to be implemented by the user of this library */
extern void writeSector(int* Sector, int SectorElements, FILE *OutputFile);

/* returns the pointer to Sector array in which states are generated in at the moment 
 * Calling getSector() somewhere else as out of function writeStateToFile() and during state generation process 
 * will cause an error message.
 * Sector has as many elements as specified by NumberOfOperators in function writeStateToFile() */
extern int* getSector();

/* function generate_Matrix_calculation
 * return value:   1 
 * DESCRIPTION:    reads out states file "InputFileName" 
 * defines FORM Locals KetVector and BraVector for every combination of states 
 * followed by a call of a FORM procedure which name is defined in "MatrixCalculationProcedure" */
extern int generate_Matrix_calculation(char* InputFileName, char* OutputFilename);

/* function createMathematicaFile
 * DESCRIPTION:    creates a the beginning of a methematica file with the name given by FileName
 * containing the definition of the excitaiton numbers and mode indices
 * followed by a call of a FORM procedure which name is defined in "MatrixCalculationProcedure" 
 * and an equatio9n for checking the level matching condition 
 * the file is not readeble for mathematica and must be completed by the FORM programm calculateMatrix.frm */
extern void createMathematicaFile(char* ModeNames[], int* ExcNumbers, int NumberOfModes, char* FileName);

/* function plotn
 * DESCRIPTION:    writes the first "number" characters of the string "Zeichenkette" to file "Output" 
 * It is the users responsibility to ensure that the file pointed by "Output" is opened.
 * If the string contains less than "number" characters the whole string is written to the file "Output". */
extern void plotn(char* Zeichenkette, int number, FILE *Output);

/* function findFirstNumberInString
 * DESCRIPTION:    returns the index of the first digit in string "Zeichenkette"
 * index counting starts with zero
 * if there is non or the string pointer is NULL the value of -1 is returned */
extern int findFirstNumberInString(char* Zeichenkette);

#endif
