/*                                               *****************************************************************
//                                               * FILE:        recycleInput.h                                   *
//                                               * written by:  Alexander Hentschel                              *
//                                               *              Humboldt University Berlin                       *
//                                               *              Alexander.Hentschel@physik.hu-berlin.de          *
//                                               *              July 2006                                        *
//                                               ****************************************************************/

#ifndef RECYCLE_INPUT_H
#define RECYCLE_INPUT_H

/* defines maximal number of Operators which are possible
 * caution:
 *   MAXOP has to be greater or equal to the number of elements in DefOperatorOrder below */
#define MAXOP 16

/* if names of mode indices aren't specified in define_states.def DefaultModeName defines the 
 * dummy name of indices going to be created by the function checkInput() */
#define DefaultModeName "n"

/* in the Mathematica input file names of the excitation numbers are generated 
 * DefaultExcitationName provides the default mode name of the excitation numbers */
#define DefaultExcitationName "x"

/* the number of states which are going to be created is calculated before the creation process
 * if the number of states is greater than about 2.14E9 states (upper bound of integer value) an error is caused
 * if AllowMoreThan2E9States is defined, the passing of this boundary doesn't cause the programm to terminate 
 * but a warning is printed 
 * in this case the value returned by the function calculateNumberOfStates() is wrong since integer overflow */
/* #define AllowMoreThan2E9States */

/* defines natural order of creation Operators, the operator names have to be surrounded by quotation marks */
#define DefOperatorOrder {"Theta4","Theta3","Theta2","Theta1","Eta4","Eta3","Eta2","Eta1","Beta4","Beta3","Beta2","Beta1","Alpha4","Alpha3","Alpha2","Alpha1"}; 

/* defines bosonic creation Operators, has to be a subset of DefOperatorOrder */
#define BosOperators 	 {"Beta4","Beta3","Beta2","Beta1","Alpha4","Alpha3","Alpha2","Alpha1"}; 


/* used function variables: 
 *	NumberOfOperators     - Numbers of Operators (listed in array UnorderedOp or Operators)
 *	NumberOfModes 	      - Numbers of Modes
 *	UnorderedOp	      - Array of strings with the names of used operators
 *	Operators	      - Array of strings with the names of used operators 
 *	ExcNumbers	      - Array of integer containing the excitation-number for each mode
 *	NumberOfExcElements   - Integer with the number of elements stored in ExcNumbers
 *	Modes		      - Array if strings containing the name of the mode indices or a NULL-Pointer
 *	NumberOfModeElements  - Integer with the number of elements stored in Modes
 *	Sector		      - Array of integer for each operator (or NULL-pointer) containing the number 
 *			        of excitations which are distributed to the operator
 *	NumbeOfSectorElements - Integer with the number of elements stored in Sector
 */

/* function today()
 * return value: String which is containing the current date and time at run time  */
extern char* today(void);

/* function OrderOperators()
 * return value: number of operators after removing duplicates
 * DESCRIPTION:
 * sorts operators given in array UnorderedOp in the order defined by DefOperatorOrder
 * If duplicates appear, a warning is printed and they are removed 
 * appearence of unknown operators causes an error message */
extern int OrderOperators(char* UnorderedOp[], int NumberOfOperators);

/* function getIndexOfFirstBosOperator()
 * return value: index of first bosonic operator in array Operators
 * DESCRIPTION:
 * function tests if all operators specified as bosonic ones by BosOperators are listed in DefOperatorOrder
 * searches for the first bosonic operator in array Operators and returns its index
 * if no bosonic operator is found the value of "NumberOfOperators" is returned
 * if BosOperators isn't defined a warning is printed, an it is assumed that all operators are fermionic */
extern int getIndexOfFirstBosOperator(char* Operators[], int NumberOfOperators);


/* function checkInput()
 * return value: pointer to Modes (array of strings as specified above)
 * DESCRIPTION:
 * checks consistency of the input: 	if number of Elements in Modes agrees with number og given Excitations 
 *					that no names of mode indices are listed more than once in array Modes ("ModeIndices")
 * if no mode indices are given the Modes-pointer is expected to be NULL, in this case an appropriate number of 
 * index names is generated using DefaultModeName as a dummy name and stored in an array. The pointer to this array is returned.  */
extern char** checkExitations_and_Modes(char* Modes[],int NumberOfModeElements, int ExcNumbers[], int NumberOfExcElements, int NumberOfOperators);

/* function freeUsedMem()
 * return value: NULL-pointer;
 * DESCRIPTION:
 * in case the function checkInput() generated names of mode indices, freeUsedMem frees the memory used for this array
 * and returns a NULL-pointer */
extern char** freeUsedMem(char* Modes[], int NumberOfModes);

/* function checkSector()
 * function checkInput(),OrderOperators() have to be ran before
 * DESCRIPTION:
 * checks consistency of Sector: if number excitations egrees with those given by ExcNumbers
 *				 if it contains an (non negative) element for every operator 
 * If operator order has been changed by OrderOperators() the sector will be reordered appropriately 
 * If Sector-pointer is NULL nothing is done */
extern void checkSector(int* Sector, int NumbeOfSectorElements, int NumberOfOperators, int NumberOfModes);

/* function reorderSecor()
 * function checkInput(),OrderOperators() have to be ran before
 * DESCRIPTION:
 * If operator order has been changed by OrderOperators() the sector will be reordered appropriately 
 * If Sector-pointer is NULL nothing is done */
extern void reorderSecor(int* Sector, int NumberOfOperators);

#endif
