/*                                               *****************************************************************
//                                               * FILE:        states.c                                         *
//                                               * written by:  Alexander Hentschel                              *
//                                               *              Humboldt University Berlin                       *
//                                               *              Alexander.Hentschel@physik.hu-berlin.de          *
//                                               *              November 2006                                    *
//                                               ****************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <limits.h>
 
#include "define_states.def"
#include "generate_states.h"
#include "recycleInput.h"
#include "generateSectors.h"

#define VERSION "1.02.02.00"
#define MaxOpLength 50
#define MaxSizeOfMathematiceFile_inGB 2
#define ShellScriptName "RunMatrixCalculation"
#if ((defined MathematicaFile) && (!defined MatrixFile))
   #define MatrixFile "matrix_calculation.load"
#endif

/* name of file which will be imported into FORM
 * import file for calculating the effective Hamiltonian
 * it will contain definitions and control statements specialised to the special structure of effective Hamiltonian */
#define FormFileII "define_Hamiltonian.load"

/* name of file which will be imported into FORM
 * import file to calculate the Hamiltonian matrix by FORM
 * it will contain definitions and control statements specialised to the special structure of the states */
#define FormFile "define_states.load"

/* name of file in which the FORM statements are stored to calculate the matrix
 * in case there is no such definition given the file is named "matrix_calculation.load"
 * this statemant is only necessary if the matrix calculation should be done analytical 
 * (so if MathematicaFile is defined) otherwise this statement is ignored  */
#define MatrixFile "matrix_calculation.load"

/* name of file in which the beginning of the Mathematica export file with name 'MathematicaFile' is stored */
#define Begin_of_MathematicaFile "Hamiltonian.begin"

#if ((!defined DefineCharges) && (defined UseOperators))
   char* UOp[] = UseOperators;
#else
   char** UOp = NULL;
#endif 
#ifdef DefineCharges
  int Charges[] = DefineCharges;
  Type_Mixing_Sectors *Mixing_Sectors;
#endif
#ifdef ModeIndices
  char* Modes[] = ModeIndices;
#else
  char **Modes = NULL;
#endif
#ifdef HamiltonianSector
  int Sector[] = HamiltonianSector;
#else
  int* Sector = NULL;
#endif
int ExcNumbers[] = ExcitationNumbers;
int NumberOfModes=0;
int NumberOfOperators=0;
int NumberOfStates=0;

void prepareHamiltonianDef(char* Operators[], int NumberOfOperators, char* Modes[] ,int NumberOfModes, char* NAME){
 int i;
 int FirstBosOp = getIndexOfFirstBosOperator(Operators,NumberOfOperators);
 FILE *f =  fopen(NAME,"w+");;
 printf("   MESSAGE: preparing FORM definition file \"%s\".\n",NAME);
 fprintf(f,"@*                                              *****************************************************************\n");
 fprintf(f,"@*                                              * FILE:        %s                          *\n",NAME);
 fprintf(f,"@*                                              * computer generated output file by C-programm states.c         *\n");
 fprintf(f,"@*                                                               on %s", today());
 fprintf(f,"@*                                              *****************************************************************\n\n");
 #ifdef VERBOSE
     fprintf(f,"#define VERBOSE\n");
 #endif
 fprintf(f,"#define StatesFile \"%s\"\n",StatesFile);
 fprintf(f,"#define NumberOfModes \"%d\"\n",NumberOfModes);
 fprintf(f,"#define NumberOfOperators \"%d\"\n",NumberOfOperators);
 fprintf(f,"#define NumberOfStates \"%d\"\n",NumberOfStates);
 fprintf(f,"\n");
 /* define a preprocessor variable for every operator */
 #ifndef HamiltonianSector
    for(i=0; i<NumberOfOperators; i++) fprintf(f,"#define %s\n",Operators[i]); fprintf(f,"\n");
 #endif
 #ifdef HamiltonianSector
    fprintf(f,"#define HamiltonianSectorDefined\n");
    for(i=0; i<NumberOfOperators; i++) fprintf(f,"#define %s \"%d\"\n",Operators[i],Sector[i]); fprintf(f,"\n");
 #endif
 /* define operators and states as noncommmuting functions in appropriate operator-state order */
 fprintf(f,"NFunction ");
 for(i=(NumberOfOperators-1); i>0; i--) fprintf(f,"Op%sm,Op%sp,",Operators[i],Operators[i]);
 fprintf(f,"Op%sm,Op%sp;\n",Operators[0],Operators[0]);
 
 fprintf(f,"\n"); 
 /* declare all operators */
/* fprintf(f,"NFunction "); 
 for(i=0; i<(NumberOfOperators-1); i++) fprintf(f,"%sPlus,%sMinus,",Operators[i],Operators[i]); 
 fprintf(f,"%sPlus,%sMinus;\n\n",Operators[NumberOfOperators-1],Operators[NumberOfOperators-1]);  */
 /* define a set of all operators */
 fprintf(f,"Set AllOperators: "); 
 for(i=0; i<(NumberOfOperators-1); i++) fprintf(f,"Op%sp,Op%sm,",Operators[i],Operators[i]); 
 fprintf(f,"Op%sp,Op%sm;\n\n",Operators[NumberOfOperators-1],Operators[NumberOfOperators-1]); 
 /* define set of all bosonic / fermionic operators listed in UseOperators 					*/
 if (FirstBosOp == NumberOfOperators) fprintf(f,"#define NoBosonicOperators\n");     
 if (FirstBosOp < NumberOfOperators)  {  
 	 fprintf(f,"Set BosOperators:  "); 		/* define set of all bosonic operators listed in UseOperators */
 	 for(i=FirstBosOp; i<(NumberOfOperators-1); i++) fprintf(f,"Op%sp,Op%sm,",Operators[i],Operators[i]);
	 fprintf(f,"Op%sp,Op%sm;\n",Operators[NumberOfOperators-1],Operators[NumberOfOperators-1]);     
 }
 if (FirstBosOp==0) fprintf(f,"#define NoFermionicOperators\n"); 
 if (FirstBosOp > 0) {  
 	 fprintf(f,"Set FermOperators: ");		 /* define set of all fermionic operators listed in UseOperators */
 	 for(i=0; i<(FirstBosOp-1); i++) fprintf(f,"Op%sp,Op%sm,",Operators[i],Operators[i]);
	 fprintf(f,"Op%sp,Op%sm;\n",Operators[FirstBosOp-1],Operators[FirstBosOp-1]);   
 }
 fclose(f);
}  /* end of function prepareHamiltonianDef */

void prepareFormDef(char* Operators[], int NumberOfOperators, char* Modes[] ,int NumberOfModes, FILE *f){
 #ifdef MathematicaFile
 int i,j;
 int FirstBosOp = getIndexOfFirstBosOperator(Operators,NumberOfOperators);
 printf("   MESSAGE: preparing FORM definition file \"%s\".\n",FormFile);
 fprintf(f,"@*                                              *****************************************************************\n");
 fprintf(f,"@*                                              * FILE:        %s                               *\n",FormFile);
 fprintf(f,"@*                                              * computer generated output file by C-programm states.c         *\n");
 fprintf(f,"@*                                                               on %s", today());
 fprintf(f,"@*                                              *****************************************************************\n\n");
 #ifdef VERBOSE
     fprintf(f,"#define VERBOSE\n");
 #endif 
 fprintf(f,"#define StatesFile \"%s\"\n",StatesFile);
 fprintf(f,"#define NumberOfModes \"%d\"\n",NumberOfModes);
 fprintf(f,"#define NumberOfOperators \"%d\"\n",NumberOfOperators);
 fprintf(f,"#define NumberOfStates \"%d\"\n",NumberOfStates);
 fprintf(f,"#define MathOutputFile \"%s\"\n",MathematicaFile);
 fprintf(f,"\n");
 fprintf(f,"#define ExBracket \"");			/* define ExBracket*/
   for(i=1; i<NumberOfModes; i++)  fprintf(f,"ExNumber%d?,",i);
   fprintf(f,"ExNumber%d?\"\n",NumberOfModes);
 for(j=1; j<=NumberOfModes; j++) { 				/* define incIndex und decIndex for all modes */
   fprintf(f,"#define incIndex%d \"",j);
      for(i=1; i<NumberOfModes; i++)  if (i==j) fprintf(f,"ExNumber%d+1,",i);
					else fprintf(f,"ExNumber%d,",i);
      if (i==j) fprintf(f,"ExNumber%d+1\"\n",NumberOfModes);
      else fprintf(f,"ExNumber%d\"\n",NumberOfModes);
   fprintf(f,"#define decIndex%d \"",j);
      for(i=1; i<NumberOfModes; i++)  if (i==j) fprintf(f,"ExNumber%d-1,",i);
					else fprintf(f,"ExNumber%d,",i);
      if (i==j) fprintf(f,"ExNumber%d-1\"\n",NumberOfModes);
      else fprintf(f,"ExNumber%d\"\n",NumberOfModes);
 } /* end of for(j=1; j<=NumberOfModes; j++)*/
 
    /* define set UsedBosSubstates of used bosonic substates, if there are none define  NoBosonicOperators 	*/
    if (FirstBosOp == NumberOfOperators) fprintf(f,"#define NoBosonicOperators\n");    
      else  { fprintf(f,"#define UsedBosSubstates \"{"); 	/* define set of all bosonic Operators listed in UseOperators 	*/
 	      for(i=FirstBosOp; i<(NumberOfOperators-1); i++) fprintf(f,"%s,",Operators[i]);
	      fprintf(f,"%s}\"\n",Operators[NumberOfOperators-1]); }
    /* define set UsedFermSubstates of used fermionic substates, if there are none define  NoFermionicOperators 		*/
    if (FirstBosOp==0) fprintf(f,"#define NoFermionicOperators\n"); /* no fermionic operators */
       else { fprintf(f,"#define UsedFermSubstates \"{");	/* define list of all fermionic Operators listed in UseOperators */
 	      for(i=0; i<(FirstBosOp-1); i++) fprintf(f,"%s,",Operators[i]);
	      fprintf(f,"%s}\"\n",Operators[FirstBosOp-1]); }
    fprintf(f,"#define OperatorBrackets \""); fprintf(f,"S%s,",Operators[NumberOfOperators-1]);
    for(i=0; i<NumberOfOperators-1; i++) fprintf(f,"%sTerms,",Operators[i]);
    fprintf(f,"%sTerms\"\n\n",Operators[NumberOfOperators-1]); 
    /* define operators and states as noncommmuting functions in appropriate operator-state order */
    for(i=(NumberOfOperators-1); i>=0; i--) 			
       fprintf(f,"NFunction S%s,Op%sm,Op%sp,Bra%s;\n",Operators[i],Operators[i],Operators[i],Operators[i]);
    fprintf(f,"\n"); 
    /* define collect (commuting) functions, which getting terms produced by one operator flavour as argument 	*/
    fprintf(f,"CFunction ");			
    for(i=0; i<NumberOfOperators-1; i++) fprintf(f,"%sTerms,",Operators[i]);
       fprintf(f,"%sTerms;\n",Operators[NumberOfOperators-1]); 
    fprintf(f,"\n");
    /* define set of all bosonic / fermionic operators listed in UseOperators 					*/
    if (FirstBosOp < NumberOfOperators)  {  
 	 fprintf(f,"Set BosOperators:  "); 		/* define set of all bosonic operators listed in UseOperators */
 	 for(i=FirstBosOp; i<(NumberOfOperators-1); i++) fprintf(f,"Op%sp,Op%sm,",Operators[i],Operators[i]);
	 fprintf(f,"Op%sp,Op%sm;\n",Operators[NumberOfOperators-1],Operators[NumberOfOperators-1]);     
 	 fprintf(f,"Set BosCreators:   "); 		/* define set of all bosonic creation operators listed in UseOperators */
 	 for(i=FirstBosOp; i<(NumberOfOperators-1); i++) fprintf(f,"Op%sp,",Operators[i]);
	 fprintf(f,"Op%sp;\n",Operators[NumberOfOperators-1]);     
 	 fprintf(f,"Set BosAnhilators: "); 		/* define set of all bosonic anhilation operators listed in UseOperators */
 	 for(i=FirstBosOp; i<(NumberOfOperators-1); i++) fprintf(f,"Op%sm,",Operators[i]);
	 fprintf(f,"Op%sm;\n",Operators[NumberOfOperators-1]);     
    }
    if (FirstBosOp > 0) {  
 	 fprintf(f,"Set FermOperators: ");		 /* define set of all fermionic operators listed in UseOperators */
 	 for(i=0; i<(FirstBosOp-1); i++) fprintf(f,"Op%sp,Op%sm,",Operators[i],Operators[i]);
	 fprintf(f,"Op%sp,Op%sm;\n",Operators[FirstBosOp-1],Operators[FirstBosOp-1]);   
 	 fprintf(f,"Set FermCreators:  ");		 /* define set of all fermionic creation operators listed in UseOperators */
 	 for(i=0; i<(FirstBosOp-1); i++) fprintf(f,"Op%sp,",Operators[i]);
	 fprintf(f,"Op%sp;\n",Operators[FirstBosOp-1]);   
 	 fprintf(f,"Set FermAnhilators: ");		 /* define set of all fermionic anhilation operators listed in UseOperators */
 	 for(i=0; i<(FirstBosOp-1); i++) fprintf(f,"Op%sm,",Operators[i]);
	 fprintf(f,"Op%sm;\n",Operators[FirstBosOp-1]);   
    }
    /* define set of all fermionic substates (need only be defined in case FirstBosOp>0 )				*/
    if (FirstBosOp>0) {	fprintf(f,"Set FermStates:    ");
    			for(i=0; i<(FirstBosOp-1); i++) fprintf(f,"S%s,",Operators[i]);
    			fprintf(f,"S%s;\n",Operators[FirstBosOp-1]);
    }
    /* define set of all needed ket-vectors									*/
    fprintf(f,"Set KetVectors:    ");
       for(i=0; i<(NumberOfOperators-1); i++) fprintf(f,"S%s,",Operators[i]);
       fprintf(f,"S%s;\n",Operators[NumberOfOperators-1]);
    /* define set of all needed bra-vectors									*/
    fprintf(f,"Set BraVectors:    ");
       for(i=0; i<(NumberOfOperators-1); i++) fprintf(f,"Bra%s,",Operators[i]);
       fprintf(f,"Bra%s;\n\n",Operators[NumberOfOperators-1]);
    /* table "ModeIndices" containing indices from file "define_states.def" or if these aren't specified
     * the automatic generated indices	 									*/
   #ifdef BraVectorFlag
     fprintf(f,"Symbol %s;\n",BraVectorFlag);
   #endif
    fprintf(f,"Symbols ");
       for(i=0; i<(NumberOfModes-1); i++) fprintf(f,"%s,",Modes[i]);
       fprintf(f,"%s;\n",Modes[NumberOfModes-1]);
    fprintf(f,"Table ModeIndices(1:%d);\n",NumberOfModes);
    fprintf(f,"Fill ModeIndices(1) = ");
       for(i=0; i<(NumberOfModes-1); i++) fprintf(f,"%s,",Modes[i]);
       fprintf(f,"%s;\n",Modes[NumberOfModes-1]);
    fprintf(f,"\n\n\n@ end of file \"%s\" \n",FormFile);
 #endif
} /* end of function prepareFormDef */

void writeStateToFile_old(int* State, int NumberOfOperators, int NumberOfModes, FILE *OutputFile){
int op, mod;
  for(op=0; op<NumberOfOperators; op++){
     fprintf(OutputFile,"| ");
     for(mod=0; mod<(NumberOfModes-1); mod++){
       fprintf(OutputFile,"%d, ",State[op*NumberOfModes + mod]);
     }
     fprintf(OutputFile,"%d",State[(op+1)*NumberOfModes - 1]);
     fprintf(OutputFile," >");
  }
  fprintf(OutputFile,"\n");
} /* end of function writeStateToFile() */ 

/* this function returns true (1) if a fermionic mode is excited more than once and otherwise false (0) */
int ScanForHigherFermionicExcitations(int* State, int NumberOfOperators, int NumberOfModes, FILE *OutputFile) {
int op, mod;
int FirstBosOp = getIndexOfFirstBosOperator(UOp,NumberOfOperators);
  for(op=0; op<FirstBosOp; op++)
    for(mod=0; mod<NumberOfModes; mod++) 
       if ((State[op*NumberOfModes + mod]> 1) || (State[op*NumberOfModes + mod]< 0)) {
           printf("\nERROR: in generating states - please report bug to Alexander Hentschel ! \n");
      	   printf("       in function ScanForHigherFermionicExcitations \n\n");
    	   exit(1);
       };
  for(op=FirstBosOp; op<NumberOfOperators; op++)
    for(mod=0; mod<NumberOfModes; mod++) 
       if (State[op*NumberOfModes + mod]< 0) {
           printf("\nERROR: in generating states - please report bug to Alexander Hentschel ! \n");
      	   printf("       in function ScanForHigherFermionicExcitations \n\n");
    	   exit(1);
       };
  return(0); 
} /* end of function ScanForHigherFermionicExcitations() */

void writeStateToFile(int* State, int NumberOfOperators, int NumberOfModes, FILE *OutputFile){
  int op, mod,G;
  int FirstBosOp = getIndexOfFirstBosOperator(UOp,NumberOfOperators);
  for(op=0; op<NumberOfOperators-1; op++){
     /* calculate number of first grassmann numbers in substate */
     G=0;
     if(op<FirstBosOp) /* fermionic substate */
         for(mod=0; mod<NumberOfModes; mod++) G+= State[op*NumberOfModes + mod];
     fprintf(OutputFile,"S%s(%d,",UOp[op],G);
     for(mod=0; mod<(NumberOfModes-1); mod++) fprintf(OutputFile,"%d,",State[op*NumberOfModes + mod]);
     fprintf(OutputFile,"%d",State[(op+1)*NumberOfModes - 1]);
     fprintf(OutputFile,")*");
  } /* now op = NumberOfOperators-1 */
  G=0;
  if(op<FirstBosOp) /* fermionic substate */
      for(mod=0; mod<NumberOfModes; mod++) G+= State[op*NumberOfModes + mod];
  fprintf(OutputFile,"S%s(%d,",UOp[NumberOfOperators-1],G);
  for(mod=0; mod<(NumberOfModes-1); mod++) fprintf(OutputFile,"%d,",State[(NumberOfOperators-1)*NumberOfModes + mod]);
  fprintf(OutputFile,"%d",State[NumberOfOperators*NumberOfModes - 1]);
  fprintf(OutputFile,")");
  fprintf(OutputFile,"\n");
  if (ScanForHigherFermionicExcitations(State,NumberOfOperators,NumberOfModes,OutputFile)) fprintf(OutputFile," @ ");
} /* end of function writeStateToFile() */ 

void writeSector(int* Sector, int SectorElements, FILE *OutputFile){
int i;  
  fprintf(OutputFile,"@ Sector: ");
  for(i=0; i<(SectorElements-1);i++) fprintf(OutputFile,"%d-",Sector[i]);
  fprintf(OutputFile,"%d\n",Sector[SectorElements-1]);
} /* end of function writeSector() */

void writeShellScript(char* NAME){
#ifdef MathematicaFile
 FILE *FilePointer;
 FilePointer = fopen(NAME,"w+");
  fprintf(FilePointer,"#!/bin/sh\n\n");
  fprintf(FilePointer,"#*                 *****************************************************************\n");
  fprintf(FilePointer,"#*                 * FILE:        %s                             *\n",NAME);
  fprintf(FilePointer,"#*                 * computer generated output file by C-programm states.c         *\n");
  fprintf(FilePointer,"#*                                   on %s", today());
  fprintf(FilePointer,"#*                 *****************************************************************\n\n");
  fprintf(FilePointer,"# internal shell variables\n"); 
  fprintf(FilePointer,"    actualpath=$(pwd)\n"); 
  fprintf(FilePointer,"    pathMatrixCalculation=calculateMatrix\n"); 
  fprintf(FilePointer,"    pathCFiles=generateStates\n"); 
  fprintf(FilePointer,"    pathHamiltonianCalculation=generateHamiltonian\n"); 
  fprintf(FilePointer,"    pathOutputFiles=output\n\n");
  
  fprintf(FilePointer,"    ToLink0=effective_Hamiltonian.def\n");
  fprintf(FilePointer,"    Source0=effective_Hamiltonian.def\n");
  fprintf(FilePointer,"    ToLink1=Hamiltonian.begin\n");
  fprintf(FilePointer,"    Source1=Hamiltonian.begin\n");
  fprintf(FilePointer,"    ToLink2=%s\n",FormFile);
  fprintf(FilePointer,"    Source2=%s\n",FormFile);
  fprintf(FilePointer,"    ToLink3=%s\n",StatesFile);
  fprintf(FilePointer,"    Source3=%s\n",StatesFile);
  fprintf(FilePointer,"    ToLink4=%s\n",MatrixFile);
  fprintf(FilePointer,"    Source4=%s\n",MatrixFile);
  fprintf(FilePointer,"    ToLink5=Hamiltonian.h\n");
  fprintf(FilePointer,"    Source5=Hamiltonian.h\n\n");
 
  fprintf(FilePointer,"   cd ../$pathMatrixCalculation/\n"); 
  fprintf(FilePointer,"   rm -f *.tmp %s\n",MathematicaFile); 
  fprintf(FilePointer,"   rm -f $ToLink0 $ToLink1 $ToLink2 $ToLink3 $ToLink4 $ToLink5\n");
  fprintf(FilePointer,"   ln -s ../$pathHamiltonianCalculation/$Source5 $ToLink5\n");
  fprintf(FilePointer,"   ln -s $actualpath/$Source0 $ToLink0\n");
  fprintf(FilePointer,"   ln -s $actualpath/$Source1 $ToLink1\n");
  fprintf(FilePointer,"   ln -s $actualpath/$Source2 $ToLink2\n");
  fprintf(FilePointer,"   ln -s $actualpath/$Source3 $ToLink3\n");
  fprintf(FilePointer,"   ln -s $actualpath/$Source4 $ToLink4\n");
  fprintf(FilePointer,"   form calculateMatrix.frm || exit\n"); 
  fprintf(FilePointer,"   cat %s > %s\n",Begin_of_MathematicaFile,MathematicaFile); 
  fprintf(FilePointer,"   cat StatesEquation.tmp >> %s\n",MathematicaFile); 
  fprintf(FilePointer,"   cat Matrix_Mathematica.tmp >> %s\n",MathematicaFile); 
  fprintf(FilePointer,"   cat MathClosure.tmp >> %s\n",MathematicaFile); 
  fprintf(FilePointer,"   rm -f $ToLink0 $ToLink1 $ToLink2 $ToLink3 $ToLink4\n");
  fprintf(FilePointer,"   rm -f *.tmp *~ %s \n",NAME);
  #if ((defined mixingSectorsFile) && (defined DefineCharges))
      fprintf(FilePointer,"   mv ../$pathCFiles/%s ../$pathOutputFiles/%s \n",mixingSectorsFile,mixingSectorsFile);
  #endif
  fprintf(FilePointer,"   mv -b %s ../$pathOutputFiles/. \n",MathematicaFile);
  fprintf(FilePointer,"   ln -f -s $pathOutputFiles/%s ../%s\n",MathematicaFile,MathematicaFile);
  #ifndef VERBOSE
    fprintf(FilePointer,"# remove temporary files\n"); 
    fprintf(FilePointer,"   cd ../$pathOutputFiles/ \n");
    fprintf(FilePointer,"   rm -f Hamiltonian.begin %s %s %s\n",FormFile,FormFileII,MatrixFile);
  #endif
  fclose(FilePointer);
#endif
} /* end of function writeShellScript */

/* function generate_apropriate_Sectors(FILE *StatesFilePointer)
 * return value: number of generated states
 * DESCRIPTION:
 * if 'HamiltonianSector' is defined the states of given sector are generated
 * if 'DefineCharges' is defined all states with given charge are generated
 * appearence of unknown operators causes an error message */
int generate_apropriate_Sectors(char* NAME) {
 FILE* StatesFilePointer = fopen(NAME,"w+");
 fprintf(StatesFilePointer,"@*                                              *****************************************************************\n");
 fprintf(StatesFilePointer,"@*                                              * FILE:        %s                                      *\n",NAME);
 fprintf(StatesFilePointer,"@*                                              * computer generated output file by C-programm states.c         *\n");
 fprintf(StatesFilePointer,"@*                                                               on %s", today());
 fprintf(StatesFilePointer,"@*                                              *****************************************************************\n\n");
 #ifdef HamiltonianSector
    #if (!defined UseOperators)
        printf("\nERROR: Please specify the corresponding operators to 'HamiltonianSector' by defining 'UseOperators'.\n\n"); exit(1);
    #endif 
    check_consistency_of_Sector_and_ExcNumbers(ExcNumbers, Sector, NumberOfOperators,NumberOfModes);
    NumberOfStates = gen_States_of_One_Sector(ExcNumbers, Sector, NumberOfOperators,getIndexOfFirstBosOperator(UOp,NumberOfOperators), NumberOfModes, StatesFilePointer);
    if (NumberOfStates == 0) { /* keine Zustaende generiert */
       printf("\nERROR: No states generated! Please compare definition of \"HamiltonianSector\"\n"); 
       printf("       with those of \"ExcNumbers\" if they allow generation of non vanishing\n");
       printf("       states. Calculation is terminated. \n");
       printf("       Error in function gen_States_of_One_Sector. \n\n");
       exit(1);
    }
 #endif /* endif of ifdef HamiltonianSector */
 #ifndef HamiltonianSector
   { /* bracket 1 */
    #ifdef DefineCharges
       int i,TotalExcitations =0;
       Type_Konf *SectorList;
       #ifdef UseOperators
          printf("   WARNING: Both 'UseOperators' and 'DefineCharges' are defined, but contributing operators\n");
          printf("            are calculated automatically on base of the given charges.\n");
          printf("            Operators given in 'UseOperators' are ignored.\n");
       #endif
       for(i=0; i<NumberOfModes; TotalExcitations += ExcNumbers[i++]);
       define_default_Operator_charges();
       Mixing_Sectors = generate_Sectors(Charges, TotalExcitations);
       compress_Sectors(Mixing_Sectors);
       SectorList = Mixing_Sectors->SectorList;
       if(SectorList == NULL){printf("\n   ERROR: no states found with given type of charge. Calculation terminated.\n\n"); exit(1);}
       #ifdef mixingSectorsFile
          write_Sectors_to_File(Mixing_Sectors,mixingSectorsFile);
       #endif
       NumberOfOperators = OrderOperators(Mixing_Sectors->OperatorList, Mixing_Sectors->NumberOfOperators); printf("\n");
       UOp = Mixing_Sectors->OperatorList;
       #ifdef VERBOSE
	      write_Sectors_to_Screen(Mixing_Sectors);
       #endif
       while(SectorList != NULL){
          reorderSecor(SectorList->Sector, Mixing_Sectors->NumberOfOperators);
	  writeSector(SectorList->Sector, NumberOfOperators, StatesFilePointer);
          NumberOfStates += gen_States_of_One_Sector(ExcNumbers, SectorList->Sector, NumberOfOperators,
	  			 getIndexOfFirstBosOperator(Mixing_Sectors->OperatorList,NumberOfOperators), 
				 NumberOfModes, StatesFilePointer);
          SectorList = SectorList->next;
       }  /* end of while */
            printf("   MESSAGE: %d mixing sectors found.\n",Mixing_Sectors->NumberOfSectors);
       #ifdef mixingSectorsFile
            printf("            Sectors written to file \"%s\".\n",mixingSectorsFile);
       #endif
    #endif /* endif of ifndef DefineCharges */    
    #ifndef DefineCharges
      double mem;
      #ifndef UseOperators
          printf("\nERROR: Please specify contributing operators by defining 'UseOperators'.\n\n"); exit(1);
      #endif
      printf("   MESSAGE: %d fermionic operator(s) detected\n",getIndexOfFirstBosOperator(UOp,NumberOfOperators));
      NumberOfStates =
         calculateNumberOfStates(ExcNumbers,NumberOfOperators,getIndexOfFirstBosOperator(UOp,NumberOfOperators),NumberOfModes);   
      printf("   MESSAGE: need to generate %d states\n",NumberOfStates);
      mem = (NumberOfStates / 8454.1 * NumberOfModes / 5 * NumberOfOperators / 4);
      printf("            approximately %dMB of data\n",((int) mem));
      NumberOfStates = gen_all_States(	ExcNumbers, NumberOfOperators,getIndexOfFirstBosOperator(UOp,NumberOfOperators),
     					NumberOfModes, StatesFilePointer);
    #endif /* endif of ifdef DefineCharges */
  } /* bracket 1 */
 #endif /* endif of ifndef HamiltonianSector */
 printf("   MESSAGE: %d states written to file \"%s\".\n",NumberOfStates ,StatesFile);
 fclose(StatesFilePointer); 
 return(0);
} /* end of generate_States_of_given_charge() */

int main() {
 #ifdef MathematicaFile
    FILE *FormFilePointer;
    double mem;
 #endif
 printf(" *****************************************************************************\n"); 
 printf(" *                  you are running ABAKUS version %s                *\n",VERSION); 
 printf(" *****************************************************************************\n"); 
 printf("\n GENERATING STATES (done by C-program states.c):\n"); 
 #if ((defined DefineCharges) && (defined HamiltonianSector))
        printf("\nERROR: Please define either 'HamiltonianSector' or 'DefineCharges'.\n\n"); exit(1);
 #endif
 NumberOfModes =  (int) sizeof(ExcNumbers)/sizeof(int);
 NumberOfOperators = OrderOperators(UOp, sizeof(UOp)/sizeof(char*)); /* Error if too much Operators are used */
 #ifndef ModeIndices 
    Modes = 
 #endif
 checkExitations_and_Modes(Modes,sizeof(Modes)/sizeof(char*),ExcNumbers,NumberOfModes,NumberOfOperators);
 checkSector(Sector, sizeof(Sector)/sizeof(int), NumberOfOperators, NumberOfModes);

 #ifdef MathematicaFile
   FormFilePointer = fopen(FormFile,"w+");
 #endif
 generate_apropriate_Sectors(StatesFile);
 #ifdef MathematicaFile
    prepareFormDef(UOp,NumberOfOperators,Modes,NumberOfModes,FormFilePointer);
 #endif
 prepareHamiltonianDef(UOp,NumberOfOperators,Modes,NumberOfModes,FormFileII);
 #ifdef MathematicaFile
     mem = mem*NumberOfStates / 100; /* groesse von "MathematicaFile" in 100MB */
     if (mem > (MaxSizeOfMathematiceFile_inGB*10)) { 
	printf("\nERROR: Size of Mathematica export file \"%s\" is \n",MathematicaFile);
	printf("       passing upper bound of %dGB (expected size approx. %d GB)!\n",MaxSizeOfMathematiceFile_inGB,((int) mem/10));
	printf("       Try the direct numerical calculation.\n");
	printf("       Calculation terminated. \n\n");
	exit(1);
     }
     createMathematicaFile(Modes,ExcNumbers,NumberOfModes,Begin_of_MathematicaFile);
     fclose(FormFilePointer); 
 #endif
 freeUsedMem(Modes,NumberOfModes);
 #ifdef MathematicaFile
    generate_Matrix_calculation(StatesFile,MatrixFile);
 #endif
 writeShellScript(ShellScriptName);
 return(0);
} /* end of main */

